@@ -1,6 +1,6 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2014-2015 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -46,12 +46,12 @@ struct _LiInstallerPrivate
 	LiPackageGraph *pg;
 	LiPackage *pkg;
 	LiPkgCache *cache;
-	GHashTable *foundations;
 
 	gchar *fname;
 	GMainLoop *loop;
 	GError *proxy_error;
-	LiProxyInstaller *bus_proxy;
+	LiProxyManager *bus_proxy;
+	guint bus_watch_id;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (LiInstaller, li_installer, G_TYPE_OBJECT)
@@ -82,7 +82,6 @@ li_installer_finalize (GObject *object)
 	g_object_unref (priv->mgr);
 	g_object_unref (priv->pg);
 	g_object_unref (priv->cache);
-	g_hash_table_unref (priv->foundations);
 	if (priv->pkg != NULL)
 		g_object_unref (priv->pkg);
 	g_free (priv->fname);
@@ -91,6 +90,8 @@ li_installer_finalize (GObject *object)
 		g_error_free (priv->proxy_error);
 	if (priv->bus_proxy != NULL)
 		g_object_unref (priv->bus_proxy);
+	if (priv->bus_watch_id != 0)
+		g_bus_unwatch_name (priv->bus_watch_id);
 
 	G_OBJECT_CLASS (li_installer_parent_class)->finalize (object);
 }
@@ -106,7 +107,6 @@ li_installer_init (LiInstaller *inst)
 	priv->mgr = li_manager_new ();
 	priv->pg = li_package_graph_new ();
 	priv->cache = li_pkg_cache_new ();
-	priv->foundations = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	priv->loop = g_main_loop_new (NULL, FALSE);
 
 	/* connect signals */
@@ -138,69 +138,6 @@ li_installer_package_graph_stage_changed_cb (LiPackageGraph *pg, LiPackageStage
 					stage, id);
 }
 
-/**
- * li_installed_dep_is_installed:
- */
-static LiPkgInfo*
-li_installer_find_satisfying_pkg (GList *pkglist, LiPkgInfo *dep)
-{
-	GList *l;
-	const gchar *dep_name;
-	const gchar *dep_version;
-	LiVersionFlags dep_vrel;
-	LiPkgInfo *res_pki = NULL;
-
-	if (pkglist == NULL)
-		return NULL;
-
-	dep_name = li_pkg_info_get_name (dep);
-	dep_version = li_pkg_info_get_version (dep);
-	dep_vrel = li_pkg_info_get_version_relation (dep);
-
-	for (l = pkglist; l != NULL; l = l->next) {
-		const gchar *pname;
-		LiPkgInfo *pki = LI_PKG_INFO (l->data);
-
-		pname = li_pkg_info_get_name (pki);
-		if (g_strcmp0 (dep_name, pname) == 0) {
-			gint cmp;
-			const gchar *pver;
-			/* we found something which has the same name as the software we are looking for */
-			pver = li_pkg_info_get_version (pki);
-			if (dep_version == NULL) {
-				/* any version satisfies this dependency - so we are happy already */
-				res_pki = pki;
-				goto out;
-			}
-
-			/* now verify that its version is sufficient */
-			cmp = li_compare_versions (pver, dep_version);
-			if (((cmp == 1) && (dep_vrel & LI_VERSION_HIGHER)) ||
-				((cmp == 0) && (dep_vrel & LI_VERSION_EQUAL)) ||
-				((cmp == -1) && (dep_vrel & LI_VERSION_LOWER))) {
-				/* we are good, the found package satisfies our requirements */
-
-				res_pki = pki;
-				goto out;
-			} else {
-				g_debug ("Found %s (%s), skipping because version does not satisfy requirements(%i#%s).", pname, pver, dep_vrel, dep_version);
-			}
-		}
-	}
-
-out:
-	if (res_pki != NULL) {
-		/* update the version of the dependency to what we found */
-		li_pkg_info_set_version (dep,
-							li_pkg_info_get_version (res_pki));
-
-		/* update the version restrictions of the found package - kind of hackish to do it here, but very convenient */
-		li_pkg_info_set_version_relation (res_pki, dep_vrel);
-	}
-
-	return res_pki;
-}
-
 /**
  * li_installer_add_dependency_remote:
  */
@@ -255,7 +192,7 @@ li_installer_find_dependency_embedded_single (LiInstaller *inst, GNode *root, Li
 	}
 
 	embedded = li_package_get_embedded_packages (pkg);
-	epki = li_installer_find_satisfying_pkg (embedded, dep_pki);
+	epki = li_find_satisfying_pkg (embedded, dep_pki);
 	if (epki == NULL) {
 		g_set_error (error,
 			LI_INSTALLER_ERROR,
@@ -349,50 +286,17 @@ li_installer_find_dependency_embedded (LiInstaller *inst, GNode *child, LiPkgInf
 	return ret;
 }
 
-/**
- * li_installer_test_foundation_dependency:
- *
- * Check if we have a foundation dependency.
- *
- * Returns: %TRUE if dependency is satisfied, and %FALSE if it is not.
- * In case we have failed to find the dependency, error is set.
- */
-static gboolean
-li_installer_test_foundation_dependency (LiInstaller *inst, LiPkgInfo *dep_pki, GError **error)
-{
-	const gchar *pkname;
-	LiInstallerPrivate *priv = GET_PRIVATE (inst);
-
-	pkname = li_pkg_info_get_name (dep_pki);
-
-	/* check if this dependency is a foundation dependency */
-	if (!g_str_has_prefix (pkname, "foundation:"))
-		return FALSE;
-
-	if (g_hash_table_lookup (priv->foundations, pkname) != NULL) {
-		/* foundation was found, dependency is satisfied! */
-		g_debug ("Detected system dependency '%s' as satisfied.", pkname);
-		return TRUE;
-	} else {
-		g_set_error (error,
-			LI_INSTALLER_ERROR,
-			LI_INSTALLER_ERROR_FOUNDATION_NOT_FOUND,
-			_("Could not find system component: '%s'. Please install it manually."), pkname);
-		return FALSE;
-	}
-}
-
 /**
  * li_installer_check_dependencies:
  */
 static void
 li_installer_check_dependencies (LiInstaller *inst, GNode *root, GError **error)
 {
-	_cleanup_list_free_ GList *all_pkgs = NULL;
+	g_autoptr(GList) all_pkgs = NULL;
 	GError *tmp_error = NULL;
 	LiPackage *pkg;
 	LiPkgInfo *pki;
-	GPtrArray *deps;
+	g_autoptr(GPtrArray) deps = NULL;
 	guint i;
 	LiInstallerPrivate *priv = GET_PRIVATE (inst);
 
@@ -421,7 +325,7 @@ li_installer_check_dependencies (LiInstaller *inst, GNode *root, GError **error)
 		LiPkgInfo *dep = LI_PKG_INFO (g_ptr_array_index (deps, i));
 
 		/* test if we have a dependency on a system component */
-		ret = li_installer_test_foundation_dependency (inst, dep, &tmp_error);
+		ret = li_package_graph_test_foundation_dependency (priv->pg, dep, &tmp_error);
 		if (tmp_error != NULL) {
 			g_propagate_error (error, tmp_error);
 			return;
@@ -431,7 +335,7 @@ li_installer_check_dependencies (LiInstaller *inst, GNode *root, GError **error)
 			continue;
 
 		/* test if this package is already in the installed set */
-		ipki = li_installer_find_satisfying_pkg (all_pkgs, dep);
+		ipki = li_find_satisfying_pkg (all_pkgs, dep);
 		if (ipki == NULL) {
 			/* maybe we find this dependency as embedded copy? */
 			li_installer_find_dependency_embedded (inst, root, dep, &tmp_error);
@@ -465,8 +369,6 @@ li_installer_check_dependencies (LiInstaller *inst, GNode *root, GError **error)
 			}
 		}
 	}
-
-	g_ptr_array_unref (deps);
 }
 
 /**
@@ -537,7 +439,7 @@ li_installer_install_node (LiInstaller *inst, GNode *node, GError **error)
 			}
 		}
 		li_pkg_info_set_runtime_dependency (info,
-										li_runtime_get_uuid (rt));
+							li_runtime_get_uuid (rt));
 		g_object_unref (rt);
 	} else {
 		/* if the installed software does not need a runtime to run, we explicity state that */
@@ -571,110 +473,50 @@ li_installer_set_package (LiInstaller *inst, LiPackage *pkg)
 }
 
 /**
- * li_installer_update_foundations_table:
+ * li_installer_proxy_error_cb:
+ *
+ * Callback for the Error() DBus signal
  */
 static void
-li_installer_update_foundations_table (LiInstaller *inst, GError **error)
+li_installer_proxy_error_cb (LiProxyManager *mgr_bus, guint32 domain, guint code, const gchar *message, LiInstaller *inst)
 {
-	_cleanup_object_unref_ LiConfigData *fdconf = NULL;
-	GError *tmp_error = NULL;
-	GFile *file;
 	LiInstallerPrivate *priv = GET_PRIVATE (inst);
 
-	if (g_hash_table_size (priv->foundations) > 0)
-		return;
-
-	fdconf = li_config_data_new ();
-	file = g_file_new_for_path (DATADIR "/foundations.list");
-	if (g_file_query_exists (file, NULL)) {
-		li_config_data_load_file (fdconf, file, &tmp_error);
-	} else {
-		g_warning ("No foundation (system-component) was defined. Continuing without that knowledge.");
-	}
-	g_object_unref (file);
-	if (tmp_error != NULL) {
-		g_propagate_error (error, tmp_error);
-		return;
+	/* ensure no error is set */
+	if (priv->proxy_error != NULL) {
+		g_error_free (priv->proxy_error);
+		priv->proxy_error = NULL;
 	}
 
-	li_config_data_reset (fdconf);
+	g_set_error (&priv->proxy_error, domain, code, "%s", message);
+}
 
-	do {
-		_cleanup_free_ gchar *fid = NULL;
-		gchar *condition;
-		gboolean ret;
-		fid = li_config_data_get_value (fdconf, "ID");
+/**
+ * li_installer_proxy_finished_cb:
+ *
+ * Callback for the Finished() DBus signal
+ */
+static void
+li_installer_proxy_finished_cb (LiProxyManager *mgr_bus, gboolean success, LiInstaller *inst)
+{
+	LiInstallerPrivate *priv = GET_PRIVATE (inst);
 
-		/* skip invalid data */
-		if (fid == NULL)
-			continue;
-
-		condition = li_config_data_get_value (fdconf, "ConditionFileExists");
-		if (condition != NULL) {
-			ret = g_file_test (condition, G_FILE_TEST_IS_REGULAR);
-			/* skip foundation if condition was not satisfied */
-			if (!ret) {
-				g_debug ("Foundation '%s' is not installed.", fid);
-				continue;
-			}
+	if (success) {
+		/* ensure no error is set */
+		if (priv->proxy_error != NULL) {
+			g_error_free (priv->proxy_error);
+			priv->proxy_error = NULL;
 		}
-
-		/* TODO: Implement ConditionLibraryExists */
-
-		/* TODO: Create a LiPkgInfo for each foundation, to produce better (error) messages later, and
-		 * to be more verbose */
-		g_hash_table_insert (priv->foundations,
-							g_strdup (fid),
-							g_strdup (fid));
-
-	} while (li_config_data_next (fdconf));
-}
-
-/**
- * li_installer_dbus_install_local_ready_cb:
- *
- * Helper callback for install() method.
- */
-static void
-li_installer_dbus_install_local_ready_cb (GObject *source_object, GAsyncResult *res, LiInstaller *inst)
-{
-	LiInstallerPrivate *priv = GET_PRIVATE (inst);
-
-	/* ensure no error is set */
-	if (priv->proxy_error != NULL) {
-		g_error_free (priv->proxy_error);
-		priv->proxy_error = NULL;
 	}
 
-	li_proxy_installer_call_install_local_finish (priv->bus_proxy, res, &priv->proxy_error);
 	g_main_loop_quit (priv->loop);
 }
 
 /**
- * li_installer_dbus_install_ready_cb:
- *
- * Helper callback for install() method.
+ * li_installer_proxy_progress_cb:
  */
 static void
-li_installer_dbus_install_ready_cb (GObject *source_object, GAsyncResult *res, LiInstaller *inst)
-{
-	LiInstallerPrivate *priv = GET_PRIVATE (inst);
-
-	/* ensure no error is set */
-	if (priv->proxy_error != NULL) {
-		g_error_free (priv->proxy_error);
-		priv->proxy_error = NULL;
-	}
-
-	li_proxy_installer_call_install_finish (priv->bus_proxy, res, &priv->proxy_error);
-	g_main_loop_quit (priv->loop);
-}
-
-/**
- * li_installer_progress_cb:
- */
-static void
-li_installer_proxy_progress_cb (LiProxyInstaller *inst_bus, const gchar *id, gint percentage, LiInstaller *inst)
+li_installer_proxy_progress_cb (LiProxyManager *mgr_bus, const gchar *id, gint percentage, LiInstaller *inst)
 {
 	if (g_strcmp0 (id, "") == 0)
 		id = NULL;
@@ -683,6 +525,29 @@ li_installer_proxy_progress_cb (LiProxyInstaller *inst_bus, const gchar *id, gin
 					percentage, id);
 }
 
+/**
+ * li_installer_bus_vanished:
+ */
+static void
+li_installer_bus_vanished (GDBusConnection *connection, const gchar *name, LiInstaller *inst)
+{
+	LiInstallerPrivate *priv = GET_PRIVATE (inst);
+
+	/* check if we are actually running any action */
+	if (!g_main_loop_is_running (priv->loop))
+		return;
+
+	if (priv->proxy_error == NULL) {
+		/* set error, since the DBus name vanished while performing an action, indicating that something crashed */
+		g_set_error (&priv->proxy_error,
+			LI_INSTALLER_ERROR,
+			LI_INSTALLER_ERROR_INTERNAL,
+			_("The Limba daemon vanished from the bus mid-transaction, so it likely crashed. Please file a bug against Limba."));
+	}
+
+	g_main_loop_quit (priv->loop);
+}
+
 /**
  * li_installer_install:
  */
@@ -700,10 +565,10 @@ li_installer_install (LiInstaller *inst, GError **error)
 
 		if (priv->bus_proxy == NULL) {
 			/* looks like we do not yet have a bus connection, so we create one */
-			priv->bus_proxy = li_proxy_installer_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+			priv->bus_proxy = li_proxy_manager_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
 											G_DBUS_PROXY_FLAGS_NONE,
 											"org.freedesktop.Limba",
-											"/org/freedesktop/Limba/Installer",
+											"/org/freedesktop/Limba/Manager",
 											NULL,
 											&tmp_error);
 			if (tmp_error != NULL) {
@@ -711,11 +576,23 @@ li_installer_install (LiInstaller *inst, GError **error)
 				goto out;
 			}
 
-			/* Installations might take quite long, so we set an infinite connection timeout. Maybe there is a better way to solve this? */
-			g_dbus_proxy_set_default_timeout (G_DBUS_PROXY (priv->bus_proxy), G_MAXINT);
+			/* we want to know when the bus name vanishes unexpectedly */
+			if (priv->bus_watch_id == 0) {
+				priv->bus_watch_id = g_bus_watch_name (G_BUS_TYPE_SYSTEM,
+									"org.freedesktop.Limba",
+									G_BUS_NAME_WATCHER_FLAGS_NONE,
+									NULL,
+									(GBusNameVanishedCallback) li_installer_bus_vanished,
+									inst,
+									NULL);
+			}
 
 			g_signal_connect (priv->bus_proxy, "progress",
 						G_CALLBACK (li_installer_proxy_progress_cb), inst);
+			g_signal_connect (priv->bus_proxy, "error",
+						G_CALLBACK (li_installer_proxy_error_cb), inst);
+			g_signal_connect (priv->bus_proxy, "finished",
+						G_CALLBACK (li_installer_proxy_finished_cb), inst);
 		}
 
 		/* ensure no error is set */
@@ -726,20 +603,29 @@ li_installer_install (LiInstaller *inst, GError **error)
 
 		if (priv->fname != NULL) {
 			/* we install a local package, so call the respective DBus method */
-			li_proxy_installer_call_install_local (priv->bus_proxy,
-											priv->fname, NULL,
-											(GAsyncReadyCallback) li_installer_dbus_install_local_ready_cb, inst);
+			li_proxy_manager_call_install_local_sync (priv->bus_proxy,
+							priv->fname,
+							NULL,
+							&tmp_error);
+			if (tmp_error != NULL) {
+				g_propagate_error (error, tmp_error);
+				goto out;
+			}
 		} else {
-			const gchar *pkid;
-
 			/* we install package from a repository */
-			pkid = li_package_get_id (priv->pkg);
-			li_proxy_installer_call_install (priv->bus_proxy,
-										pkid, NULL,
-										(GAsyncReadyCallback) li_installer_dbus_install_ready_cb, inst);
-			g_main_loop_run (priv->loop);
+			li_proxy_manager_call_install_sync (priv->bus_proxy,
+							li_package_get_id (priv->pkg),
+							NULL,
+							&tmp_error);
+			if (tmp_error != NULL) {
+				g_propagate_error (error, tmp_error);
+				goto out;
+			}
 		}
 
+		/* if we are here, we can wait for a Finished() signal */
+		g_main_loop_run (priv->loop);
+
 		if (priv->proxy_error != NULL) {
 			g_propagate_error (error, priv->proxy_error);
 			priv->proxy_error = NULL;
@@ -749,6 +635,13 @@ li_installer_install (LiInstaller *inst, GError **error)
 		goto out;
 	}
 
+	/* ensure the graph is initialized and additional data (foundations list) is loaded */
+	li_package_graph_initialize (priv->pg, &tmp_error);
+	if (tmp_error != NULL) {
+		g_propagate_error (error, tmp_error);
+		goto out;
+	}
+
 	root = li_package_graph_get_root (priv->pg);
 	if (root->data == NULL) {
 		g_set_error (error,
@@ -758,14 +651,6 @@ li_installer_install (LiInstaller *inst, GError **error)
 		return FALSE;
 	}
 
-	/* populate the foundations registry, if not yet done */
-	li_installer_update_foundations_table (inst, &tmp_error);
-	if (tmp_error != NULL) {
-		g_propagate_prefixed_error (error, tmp_error,
-									"Could not load foundations list.");
-		goto out;
-	}
-
 	/* open the package cache */
 	li_pkg_cache_open (priv->cache, &tmp_error);
 	if (tmp_error != NULL) {
@@ -835,7 +720,7 @@ li_installer_open_file (LiInstaller *inst, const gchar *filename, GError **error
 gboolean
 li_installer_open_remote (LiInstaller *inst, const gchar *pkgid, GError **error)
 {
-	LiPackage *pkg;
+	g_autoptr(LiPackage) pkg = NULL;
 	GError *tmp_error = NULL;
 	LiInstallerPrivate *priv = GET_PRIVATE (inst);
 
@@ -850,12 +735,16 @@ li_installer_open_remote (LiInstaller *inst, const gchar *pkgid, GError **error)
 	li_package_open_remote (pkg, priv->cache, pkgid, &tmp_error);
 	if (tmp_error != NULL) {
 		g_propagate_error (error, tmp_error);
-		g_object_unref (pkg);
 		return FALSE;
 	}
 
+	/* when downloading packages from the cache, we already verified the index file,
+	 * and after download we also verify the SHA256 checksums of the index - no need
+	 * to also verify the internal signature of the IPK package (especially if we may
+	 * not have its public-key in the keyring). */
+	li_package_set_auto_verify (pkg, FALSE);
+
 	li_installer_set_package (inst, pkg);
-	g_object_unref (pkg);
 
 	return TRUE;
 }
@@ -881,7 +770,7 @@ LiTrustLevel
 li_installer_get_package_trust_level (LiInstaller *inst)
 {
 	LiInstallerPrivate *priv = GET_PRIVATE (inst);
-	return	li_package_verify_signature (priv->pkg, NULL);
+	return li_package_verify_signature (priv->pkg, NULL);
 }
 
 /**
@@ -901,6 +790,7 @@ li_installer_get_appstream_data (LiInstaller *inst)
 
 	return li_package_get_appstream_data (priv->pkg);
 }
+
 /**
  * li_installer_error_quark:
  *
