@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2014 Matthias Klumpp <matthias@tenstral.net>
  *
@@ -52,6 +52,7 @@ struct _LiManagerPrivate
 	GMainLoop *loop;
 	LiProxyManager *bus_proxy;
 	GError *proxy_error;
+	guint bus_watch_id;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (LiManager, li_manager, G_TYPE_OBJECT)
@@ -120,9 +121,9 @@ li_manager_init (LiManager *mgr)
 	priv->pkgs = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 	priv->rts = g_ptr_array_new_with_free_func (g_object_unref);
 	priv->updates = g_hash_table_new_full ((GHashFunc) li_pki_hash_func,
-									(GEqualFunc) li_pki_equal_func,
-									g_object_unref,
-									g_object_unref);
+						(GEqualFunc) li_pki_equal_func,
+						g_object_unref,
+						g_object_unref);
 	priv->loop = g_main_loop_new (NULL, FALSE);
 }
 
@@ -147,8 +148,8 @@ static GHashTable*
 li_manager_get_installed_software (LiManager *mgr, GError **error)
 {
 	GError *tmp_error = NULL;
-	GFile *fdir;
-	GFileEnumerator *enumerator = NULL;
+	g_autoptr(GFile) fdir = NULL;
+	g_autoptr(GFileEnumerator) enumerator = NULL;
 	GFileInfo *file_info;
 	GHashTable *pkgs = NULL;
 
@@ -166,51 +167,72 @@ li_manager_get_installed_software (LiManager *mgr, GError **error)
 	pkgs = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 
 	while ((file_info = g_file_enumerator_next_file (enumerator, NULL, &tmp_error)) != NULL) {
-		_cleanup_free_ gchar *path = NULL;
+		g_autoptr(GFile) fsdir = NULL;
+		g_autoptr(GFileEnumerator) senum = NULL;
+		g_autofree gchar *mpath = NULL;
+		GFileInfo *s_finfo;
+
+		if (tmp_error != NULL)
+			goto out;
+		if (g_file_info_get_is_hidden (file_info))
+			continue;
+		/* ignore the runtimes directory */
+		if (g_strcmp0 (g_file_info_get_name (file_info), "runtimes") == 0)
+			continue;
+
+		mpath = g_build_filename (LI_SOFTWARE_ROOT, g_file_info_get_name (file_info), NULL);
+
+		fsdir = g_file_new_for_path (mpath);
+		senum = g_file_enumerate_children (fsdir, G_FILE_ATTRIBUTE_STANDARD_NAME, 0, NULL, &tmp_error);
 		if (tmp_error != NULL)
 			goto out;
 
-		if (g_file_info_get_is_hidden (file_info))
-			continue;
-		path = g_build_filename (LI_SOFTWARE_ROOT,
-								 g_file_info_get_name (file_info),
-								 "control",
-								 NULL);
-		if (g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
-			_cleanup_object_unref_ GFile *ctlfile = NULL;
-			ctlfile = g_file_new_for_path (path);
-			if (g_file_query_exists (ctlfile, NULL)) {
-				LiPkgInfo *pki;
+		while ((s_finfo = g_file_enumerator_next_file (senum, NULL, &tmp_error)) != NULL) {
+			g_autofree gchar *cpath = NULL;
 
-				/* create new LiPkgInfo for an installed package */
-				pki = li_pkg_info_new ();
+			if (tmp_error != NULL)
+				goto out;
+			if (g_file_info_get_is_hidden (s_finfo))
+				continue;
 
-				li_pkg_info_load_file (pki, ctlfile, &tmp_error);
-				if (tmp_error != NULL) {
-					g_object_unref (pki);
-					goto out;
+			cpath = g_build_filename (mpath,
+						g_file_info_get_name (s_finfo),
+						"control",
+						NULL);
+
+			if (g_file_test (cpath, G_FILE_TEST_IS_REGULAR)) {
+				g_autoptr(GFile) ctlfile = NULL;
+				ctlfile = g_file_new_for_path (cpath);
+				if (g_file_query_exists (ctlfile, NULL)) {
+					LiPkgInfo *pki;
+
+					/* create new LiPkgInfo for an installed package */
+					pki = li_pkg_info_new ();
+
+					li_pkg_info_load_file (pki, ctlfile, &tmp_error);
+					if (tmp_error != NULL) {
+						g_object_unref (pki);
+						goto out;
+					}
+
+					/* do not list as installed if the software is faded */
+					if (!li_pkg_info_has_flag (pki, LI_PACKAGE_FLAG_FADED)) {
+						li_pkg_info_add_flag (pki, LI_PACKAGE_FLAG_INSTALLED);
+					}
+
+					g_hash_table_insert (pkgs,
+								g_strdup (li_pkg_info_get_id (pki)),
+								pki);
 				}
-
-				/* do not list as installed if the software is faded */
-				if (!li_pkg_info_has_flag (pki, LI_PACKAGE_FLAG_FADED)) {
-					li_pkg_info_add_flag (pki, LI_PACKAGE_FLAG_INSTALLED);
-				}
-
-				g_hash_table_insert (pkgs,
-									g_strdup (li_pkg_info_get_id (pki)),
-									pki);
 			}
 		}
 	}
 
 out:
-	g_object_unref (fdir);
-	if (enumerator != NULL)
-		g_object_unref (enumerator);
 	if (tmp_error != NULL) {
 		g_propagate_prefixed_error (error,
-						tmp_error,
-						"Error while searching for installed software:");
+					tmp_error,
+					"Error while searching for installed software:");
 		if (pkgs != NULL)
 			g_hash_table_unref (pkgs);
 		return NULL;
@@ -225,7 +247,7 @@ out:
 static void
 li_manager_update_software_table (LiManager *mgr, GError **error)
 {
-	_cleanup_object_unref_ LiPkgCache *cache = NULL;
+	g_autoptr(LiPkgCache) cache = NULL;
 	GPtrArray *apkgs;
 	GHashTable *ipkgs;
 	guint i;
@@ -265,8 +287,8 @@ li_manager_update_software_table (LiManager *mgr, GError **error)
 		/* add packages to the global list, if they are not already installed */
 		if (g_hash_table_lookup (priv->pkgs, li_pkg_info_get_id (pki)) == NULL) {
 			g_hash_table_insert (priv->pkgs,
-								g_strdup (li_pkg_info_get_id (pki)),
-								g_object_ref (pki));
+						g_strdup (li_pkg_info_get_id (pki)),
+						g_object_ref (pki));
 		}
 	}
 }
@@ -292,7 +314,7 @@ li_manager_get_software_list (LiManager *mgr, GError **error)
 }
 
 /**
- * li_manager_get_software_by_id:
+ * li_manager_get_software_by_pkid:
  *
  * Returns: (transfer none): A #LiPkgInfo or %NULL if no software was found.
  **/
@@ -321,7 +343,7 @@ li_manager_find_installed_runtimes (LiManager *mgr)
 	GFile *fdir;
 	GFileEnumerator *enumerator = NULL;
 	GFileInfo *file_info;
-	_cleanup_free_ gchar *runtime_root;
+	g_autofree gchar *runtime_root;
 	LiManagerPrivate *priv = GET_PRIVATE (mgr);
 
 	runtime_root = g_build_filename (LI_SOFTWARE_ROOT, "runtimes", NULL);
@@ -337,32 +359,24 @@ li_manager_find_installed_runtimes (LiManager *mgr)
 		goto out;
 
 	while ((file_info = g_file_enumerator_next_file (enumerator, NULL, &tmp_error)) != NULL) {
-		gchar *path;
+		g_autofree gchar *rt_path = NULL;
 		if (tmp_error != NULL)
 			goto out;
 
 		if (g_file_info_get_is_hidden (file_info))
 			continue;
-		path = g_build_filename (runtime_root,
-								 g_file_info_get_name (file_info),
-								 "control",
-								 NULL);
-		if (g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
-			gchar *rt_path;
+		rt_path = g_build_filename (runtime_root, g_file_info_get_name (file_info), NULL);
+		if (g_file_test (rt_path, G_FILE_TEST_IS_REGULAR)) {
 			gboolean ret;
 			LiRuntime *rt;
 
-			rt_path = g_build_filename (runtime_root, g_file_info_get_name (file_info), NULL);
-
 			rt = li_runtime_new ();
-			ret = li_runtime_load_directory (rt, rt_path, &tmp_error);
+			ret = li_runtime_load_from_file (rt, rt_path, &tmp_error);
 			if (ret)
 				g_ptr_array_add (priv->rts, g_object_ref (rt));
 
-			g_free (rt_path);
 			g_object_unref (rt);
 		}
-		g_free (path);
 	}
 
 
@@ -537,12 +551,12 @@ li_manager_proxy_progress_cb (LiProxyManager *mgr_bus, const gchar *id, gint per
 }
 
 /**
- * li_manager_dbus_remove_software_ready_cb:
+ * li_manager_proxy_error_cb:
  *
- * Helper callback for removeSoftware() method.
+ * Callback for the Error() DBus signal
  */
 static void
-li_manager_dbus_remove_software_ready_cb (GObject *source_object, GAsyncResult *res, LiManager *mgr)
+li_manager_proxy_error_cb (LiProxyManager *mgr_bus, guint32 domain, guint code, const gchar *message, LiManager *mgr)
 {
 	LiManagerPrivate *priv = GET_PRIVATE (mgr);
 
@@ -552,7 +566,50 @@ li_manager_dbus_remove_software_ready_cb (GObject *source_object, GAsyncResult *
 		priv->proxy_error = NULL;
 	}
 
-	li_proxy_manager_call_remove_software_finish (priv->bus_proxy, res, &priv->proxy_error);
+	g_set_error (&priv->proxy_error, domain, code, "%s", message);
+}
+
+/**
+ * li_manager_proxy_finished_cb:
+ *
+ * Callback for the Finished() DBus signal
+ */
+static void
+li_manager_proxy_finished_cb (LiProxyManager *mgr_bus, gboolean success, LiManager *mgr)
+{
+	LiManagerPrivate *priv = GET_PRIVATE (mgr);
+
+	if (success) {
+		/* ensure no error is set */
+		if (priv->proxy_error != NULL) {
+			g_error_free (priv->proxy_error);
+			priv->proxy_error = NULL;
+		}
+	}
+
+	g_main_loop_quit (priv->loop);
+}
+
+/**
+ * li_manager_bus_vanished:
+ */
+static void
+li_manager_bus_vanished (GDBusConnection *connection, const gchar *name, LiManager *mgr)
+{
+	LiManagerPrivate *priv = GET_PRIVATE (mgr);
+
+	/* check if we are actually running any action */
+	if (!g_main_loop_is_running (priv->loop))
+		return;
+
+	if (priv->proxy_error == NULL) {
+		/* set error, since the DBus name vanished while performing an action, indicating that something crashed */
+		g_set_error (&priv->proxy_error,
+			LI_INSTALLER_ERROR,
+			LI_INSTALLER_ERROR_INTERNAL,
+			_("The Limba daemon vanished from the bus mid-transaction, so it likely crashed. Please file a bug against Limba."));
+	}
+
 	g_main_loop_quit (priv->loop);
 }
 
@@ -562,11 +619,11 @@ li_manager_dbus_remove_software_ready_cb (GObject *source_object, GAsyncResult *
 gboolean
 li_manager_remove_software (LiManager *mgr, const gchar *pkgid, GError **error)
 {
-	_cleanup_free_ gchar *swpath = NULL;
+	g_autofree gchar *swpath = NULL;
 	GFile *expfile;
 	gchar *tmp;
 	GError *tmp_error = NULL;
-	_cleanup_ptrarray_unref_ GPtrArray *pkg_array = NULL;
+	g_autoptr(GPtrArray) pkg_array = NULL;
 	GFile *ctlfile;
 	LiPkgInfo *pki;
 	LiRuntime *rt;
@@ -579,21 +636,33 @@ li_manager_remove_software (LiManager *mgr, const gchar *pkgid, GError **error)
 		if (priv->bus_proxy == NULL) {
 			/* looks like we do not yet have a bus connection, so we create one */
 			priv->bus_proxy = li_proxy_manager_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
-											G_DBUS_PROXY_FLAGS_NONE,
-											"org.freedesktop.Limba",
-											"/org/freedesktop/Limba/Manager",
-											NULL,
-											&tmp_error);
+										G_DBUS_PROXY_FLAGS_NONE,
+										"org.freedesktop.Limba",
+										"/org/freedesktop/Limba/Manager",
+										NULL,
+										&tmp_error);
 			if (tmp_error != NULL) {
 				g_propagate_error (error, tmp_error);
 				goto out;
 			}
 
-			/* Installations might take quite long, so we set an infinite connection timeout. Maybe there is a better way to solve this? */
-			g_dbus_proxy_set_default_timeout (G_DBUS_PROXY (priv->bus_proxy), G_MAXINT);
+			/* we want to know when the bus name vanishes unexpectedly */
+			if (priv->bus_watch_id == 0) {
+				priv->bus_watch_id = g_bus_watch_name (G_BUS_TYPE_SYSTEM,
+									"org.freedesktop.Limba",
+									G_BUS_NAME_WATCHER_FLAGS_NONE,
+									NULL,
+									(GBusNameVanishedCallback) li_manager_bus_vanished,
+									mgr,
+									NULL);
+			}
 
 			g_signal_connect (priv->bus_proxy, "progress",
 						G_CALLBACK (li_manager_proxy_progress_cb), mgr);
+			g_signal_connect (priv->bus_proxy, "error",
+						G_CALLBACK (li_manager_proxy_error_cb), mgr);
+			g_signal_connect (priv->bus_proxy, "finished",
+						G_CALLBACK (li_manager_proxy_finished_cb), mgr);
 		}
 
 		/* ensure no error is set */
@@ -602,9 +671,15 @@ li_manager_remove_software (LiManager *mgr, const gchar *pkgid, GError **error)
 			priv->proxy_error = NULL;
 		}
 
-		li_proxy_manager_call_remove_software (priv->bus_proxy,
-										pkgid, NULL,
-										(GAsyncReadyCallback) li_manager_dbus_remove_software_ready_cb, mgr);
+		li_proxy_manager_call_remove_software_sync (priv->bus_proxy,
+						pkgid,
+						NULL,
+						&tmp_error);
+		if (tmp_error != NULL) {
+			g_propagate_error (error, tmp_error);
+			goto out;
+		}
+
 		g_main_loop_run (priv->loop);
 
 		if (priv->proxy_error != NULL) {
@@ -624,9 +699,9 @@ li_manager_remove_software (LiManager *mgr, const gchar *pkgid, GError **error)
 	if (!g_file_query_exists (ctlfile, NULL)) {
 		g_object_unref (ctlfile);
 		g_set_error (error,
-					LI_MANAGER_ERROR,
-					LI_MANAGER_ERROR_NOT_FOUND,
-					_("Could not find software: %s"), pkgid);
+				LI_MANAGER_ERROR,
+				LI_MANAGER_ERROR_NOT_FOUND,
+				_("Could not find software: %s"), pkgid);
 		return FALSE;
 	}
 	pki = li_pkg_info_new ();
@@ -643,7 +718,7 @@ li_manager_remove_software (LiManager *mgr, const gchar *pkgid, GError **error)
 	g_ptr_array_add (pkg_array, pki);
 	rt = li_manager_find_runtime_with_members (mgr, pkg_array);
 	if (rt != NULL) {
-		_cleanup_list_free_ GList *sw = NULL;
+		g_autoptr(GList) sw = NULL;
 		GList *l;
 		gboolean dependency_found = FALSE;
 
@@ -699,9 +774,9 @@ li_manager_remove_software (LiManager *mgr, const gchar *pkgid, GError **error)
 	/* now delete the directory */
 	if (!li_delete_dir_recursive (swpath)) {
 		g_set_error (error,
-					LI_MANAGER_ERROR,
-					LI_MANAGER_ERROR_REMOVE_FAILED,
-					_("Could not remove software directory."));
+				LI_MANAGER_ERROR,
+				LI_MANAGER_ERROR_REMOVE_FAILED,
+				_("Could not remove software directory."));
 		return FALSE;
 	}
 
@@ -722,11 +797,11 @@ out:
 gboolean
 li_manager_package_is_installed (LiManager *mgr, LiPkgInfo *pki)
 {
-	_cleanup_free_ gchar *pkid = NULL;
-	_cleanup_free_ gchar *path = NULL;
+	g_autofree gchar *pkid = NULL;
+	g_autofree gchar *path = NULL;
 
-	pkid = g_strdup_printf ("%s-%s",
-							li_pkg_info_get_name (pki), li_pkg_info_get_version (pki));
+	pkid = g_strdup_printf ("%s/%s",
+				li_pkg_info_get_name (pki), li_pkg_info_get_version (pki));
 	path = g_build_filename (LI_SOFTWARE_ROOT, pkid, "control", NULL);
 
 	if (g_file_test (path, G_FILE_TEST_IS_REGULAR))
@@ -744,8 +819,8 @@ static void
 li_manager_cleanup_broken_packages (LiManager *mgr, GError **error)
 {
 	GError *tmp_error = NULL;
-	GFile *fdir;
-	GFileEnumerator *enumerator = NULL;
+	g_autoptr(GFile) fdir = NULL;
+	g_autoptr(GFileEnumerator) enumerator = NULL;
 	GFileInfo *file_info;
 
 	if (!g_file_test (LI_SOFTWARE_ROOT, G_FILE_TEST_IS_DIR)) {
@@ -755,38 +830,70 @@ li_manager_cleanup_broken_packages (LiManager *mgr, GError **error)
 
 	/* get stuff in the software directory */
 	fdir = g_file_new_for_path (LI_SOFTWARE_ROOT);
-	enumerator = g_file_enumerate_children (fdir, G_FILE_ATTRIBUTE_STANDARD_NAME, 0, NULL, &tmp_error);
+	enumerator = g_file_enumerate_children (fdir,
+						G_FILE_ATTRIBUTE_STANDARD_NAME,
+						G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+						NULL,
+						&tmp_error);
 	if (tmp_error != NULL)
 		goto out;
 
 	while ((file_info = g_file_enumerator_next_file (enumerator, NULL, &tmp_error)) != NULL) {
-		_cleanup_free_ gchar *path = NULL;
+		g_autoptr(GFile) fsdir = NULL;
+		g_autoptr(GFileEnumerator) senum = NULL;
+		g_autofree gchar *mpath = NULL;
+		gint child_count = 0;
+		GFileInfo *s_finfo;
+
 		if (tmp_error != NULL)
 			goto out;
-
+		if (g_file_info_get_is_hidden (file_info))
+			continue;
 		/* the "runtimes" directory is special, never remove it */
 		if (g_strcmp0 (g_file_info_get_name (file_info), "runtimes") == 0)
 			continue;
 
-		path = g_build_filename (LI_SOFTWARE_ROOT,
-								 g_file_info_get_name (file_info),
-								 "control",
-								 NULL);
+		mpath = g_build_filename (LI_SOFTWARE_ROOT, g_file_info_get_name (file_info), NULL);
 
-		/* no control file means this is garbage, probably from a previous failed installation */
-		if (!g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
-			_cleanup_free_ gchar *tmp_path;
-			tmp_path = g_build_filename (LI_SOFTWARE_ROOT,
-									g_file_info_get_name (file_info),
-									NULL);
-			li_delete_dir_recursive (tmp_path);
+		fsdir = g_file_new_for_path (mpath);
+		senum = g_file_enumerate_children (fsdir,
+						   G_FILE_ATTRIBUTE_STANDARD_NAME,
+						   G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+						   NULL,
+						   &tmp_error);
+		if (tmp_error != NULL)
+			goto out;
+
+		while ((s_finfo = g_file_enumerator_next_file (senum, NULL, &tmp_error)) != NULL) {
+			g_autofree gchar *path = NULL;
+			if (tmp_error != NULL)
+				goto out;
+			if (g_file_info_get_is_hidden (file_info))
+				continue;
+
+			path = g_build_filename (mpath,
+						 g_file_info_get_name (s_finfo),
+						 "control",
+						 NULL);
+
+			/* no control file means this is garbage, probably from a previous failed installation */
+			if (!g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
+				g_autofree gchar *tmp_path;
+				tmp_path = g_build_filename (mpath,
+								g_file_info_get_name (file_info),
+								NULL);
+				li_delete_dir_recursive (tmp_path);
+			} else {
+				child_count++;
+			}
 		}
+
+		/* check if directory is empty, delete it in that case */
+		if (child_count == 0)
+			g_remove (mpath);
 	}
 
 out:
-	g_object_unref (fdir);
-	if (enumerator != NULL)
-		g_object_unref (enumerator);
 	if (tmp_error != NULL) {
 		g_propagate_prefixed_error (error,
 						tmp_error,
@@ -802,13 +909,13 @@ out:
 gboolean
 li_manager_cleanup (LiManager *mgr, GError **error)
 {
-	_cleanup_hashtable_unref_ GHashTable *sws = NULL;
-	_cleanup_hashtable_unref_ GHashTable *rts = NULL;
-	_cleanup_list_free_ GList *sw_list = NULL;
+	g_autoptr(GHashTable) sws = NULL;
+	g_autoptr(GHashTable) rts = NULL;
+	g_autoptr(GList) sw_list = NULL;
+	g_autoptr(GList) list = NULL;
 	GPtrArray *rt_array;
 	guint i;
 	GList *l;
-	_cleanup_list_free_ GList *list = NULL;
 	GError *tmp_error = NULL;
 	gboolean faded_sw_removed = FALSE;
 	gboolean ret = FALSE;
@@ -878,8 +985,8 @@ li_manager_cleanup (LiManager *mgr, GError **error)
 		g_free (rt_members);
 
 		g_hash_table_insert (rts,
-							g_strdup (li_runtime_get_uuid (rt)),
-							g_object_ref (rt));
+					g_strdup (li_runtime_get_uuid (rt)),
+					g_object_ref (rt));
 	}
 
 	/* remove every software from the kill-list which references a valid runtime */
@@ -925,7 +1032,7 @@ out:
 void
 li_manager_refresh_cache (LiManager *mgr, GError **error)
 {
-	_cleanup_object_unref_ LiPkgCache *cache = NULL;
+	g_autoptr(LiPkgCache) cache = NULL;
 	GError *tmp_error = NULL;
 
 	cache = li_pkg_cache_new ();
@@ -971,9 +1078,9 @@ li_manager_clear_updates_table (LiManager *mgr)
 
 	g_hash_table_unref (priv->updates);
 	priv->updates = g_hash_table_new_full ((GHashFunc) li_pki_hash_func,
-									(GEqualFunc) li_pki_equal_func,
-									g_object_unref,
-									g_object_unref);
+						(GEqualFunc) li_pki_equal_func,
+						g_object_unref,
+						g_object_unref);
 }
 
 /**
@@ -986,12 +1093,12 @@ li_manager_clear_updates_table (LiManager *mgr)
 GList*
 li_manager_get_update_list (LiManager *mgr, GError **error)
 {
-	_cleanup_object_unref_ LiPkgCache *cache = NULL;
+	g_autoptr(LiPkgCache) cache = NULL;
+	g_autoptr(GHashTable) ipkgs = NULL;
+	g_autoptr(GHashTable) apkgs = NULL;
+	g_autoptr(GList) ipkg_list = NULL;
 	GPtrArray *apkgs_list;
-	_cleanup_hashtable_unref_ GHashTable *ipkgs = NULL;
-	_cleanup_hashtable_unref_ GHashTable *apkgs = NULL;
 	guint i;
-	_cleanup_list_free_ GList *ipkg_list = NULL;
 	GList *l;
 	GError *tmp_error = NULL;
 	LiManagerPrivate *priv = GET_PRIVATE (mgr);
@@ -1027,8 +1134,8 @@ li_manager_get_update_list (LiManager *mgr, GError **error)
 		}
 
 		g_hash_table_replace (apkgs,
-							g_strdup (li_pkg_info_get_name (b_pki)),
-							g_object_ref (b_pki));
+					g_strdup (li_pkg_info_get_name (b_pki)),
+					g_object_ref (b_pki));
 	}
 
 	/* ensure we have a clean updates table */
@@ -1042,7 +1149,7 @@ li_manager_get_update_list (LiManager *mgr, GError **error)
 
 		apki = g_hash_table_lookup (apkgs, li_pkg_info_get_name (ipki));
 		/* check if we actually have a package available */
-		if (ipki == NULL)
+		if (apki == NULL)
 			continue;
 
 		if (li_compare_versions (li_pkg_info_get_version (apki), li_pkg_info_get_version (ipki)) > 0) {
@@ -1051,8 +1158,8 @@ li_manager_get_update_list (LiManager *mgr, GError **error)
 			/* we have a potential update */
 			uitem = li_update_item_new_with_packages (ipki, apki);
 			g_hash_table_insert (priv->updates,
-							g_object_ref (ipki),
-							uitem);
+						g_object_ref (ipki),
+						uitem);
 		}
 	}
 
@@ -1065,14 +1172,14 @@ li_manager_get_update_list (LiManager *mgr, GError **error)
 static void
 li_manager_remove_exported_files_by_pki (LiManager *mgr, LiPkgInfo *pki, GError **error)
 {
-	_cleanup_free_ gchar *swpath = NULL;
-	_cleanup_object_unref_ GFile *expfile = NULL;
+	g_autofree gchar *swpath = NULL;
+	g_autoptr(GFile) expfile = NULL;
 	GError *tmp_error = NULL;
 	gchar *tmp;
 
 	swpath = g_build_filename (LI_SOFTWARE_ROOT,
-								li_pkg_info_get_id (pki),
-								NULL);
+					li_pkg_info_get_id (pki),
+					NULL);
 
 	/* delete exported files */
 	/* FIXME: Move them away at first, to allow a later rollback */
@@ -1102,7 +1209,7 @@ li_manager_remove_exported_files_by_pki (LiManager *mgr, LiPkgInfo *pki, GError
 static gboolean
 li_manager_upgrade_single_package (LiManager *mgr, LiPkgInfo *ipki, LiPkgInfo *apki, GError **error)
 {
-	_cleanup_object_unref_ LiInstaller *inst = NULL;
+	g_autoptr(LiInstaller) inst = NULL;
 	GError *tmp_error = NULL;
 
 	/* prepare installation */
@@ -1137,8 +1244,8 @@ li_manager_upgrade_single_package (LiManager *mgr, LiPkgInfo *ipki, LiPkgInfo *a
 gboolean
 li_manager_apply_updates (LiManager *mgr, GError **error)
 {
-	_cleanup_list_free_ GList *updlist = NULL;
-	_cleanup_hashtable_unref_ GHashTable *ipkgs = NULL;
+	g_autoptr(GList) updlist = NULL;
+	g_autoptr(GHashTable) ipkgs = NULL;
 	GList *l;
 	GError *tmp_error = NULL;
 	LiManagerPrivate *priv = GET_PRIVATE (mgr);
@@ -1164,7 +1271,7 @@ li_manager_apply_updates (LiManager *mgr, GError **error)
 	for (l = updlist; l != NULL; l = l->next) {
 		LiPkgInfo *ipki;
 		LiPkgInfo *apki;
-		_cleanup_ptrarray_unref_ GPtrArray *rts = NULL;
+		g_autoptr(GPtrArray) rts = NULL;
 
 		LiUpdateItem *uitem = LI_UPDATE_ITEM (l->data);
 
@@ -1192,7 +1299,7 @@ li_manager_apply_updates (LiManager *mgr, GError **error)
 
 		} else {
 			guint i;
-			_cleanup_ptrarray_unref_ GPtrArray *update_rts = NULL;
+			g_autoptr(GPtrArray) update_rts = NULL;
 
 			g_debug ("Performing complex upgrade of '%s'", li_pkg_info_get_id (ipki));
 
