@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2014 Matthias Klumpp <matthias@tenstral.net>
  *
@@ -63,6 +63,7 @@ struct _LiPackagePrivate
 	LiKeyring *kr;
 	gchar *signature_data;
 	gchar *sig_fpr;
+	gboolean auto_verify;
 	LiTrustLevel tlevel;
 	GHashTable *contents_hash;
 
@@ -138,6 +139,7 @@ li_package_init (LiPackage *pkg)
 	priv->kr = li_keyring_new ();
 	priv->contents_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	priv->tlevel = LI_TRUST_LEVEL_NONE;
+	priv->auto_verify = TRUE; /* we verify the package signature by default */
 }
 
 /**
@@ -215,7 +217,7 @@ li_package_read_entry (LiPackage *pkg, struct archive* ar, GError **error)
 static gboolean
 li_package_extract_entry_to (LiPackage *pkg, struct archive* ar, struct archive_entry* e, const gchar* dest, GError **error)
 {
-	_cleanup_free_ gchar *fname = NULL;
+	g_autofree gchar *fname = NULL;
 	const gchar *cstr;
 	const gchar *link_target;
 	gchar *str;
@@ -244,9 +246,9 @@ li_package_extract_entry_to (LiPackage *pkg, struct archive* ar, struct archive_
 
 	if (g_file_test (fname, G_FILE_TEST_EXISTS)) {
 		g_set_error (error,
-				LI_PACKAGE_ERROR,
-				LI_PACKAGE_ERROR_OVERRIDE,
-				_("Could not override file '%s'. The file already exists!"), fname);
+			LI_PACKAGE_ERROR,
+			LI_PACKAGE_ERROR_OVERRIDE,
+			_("Could not override file '%s'. The file already exists!"), fname);
 		return FALSE;
 	}
 
@@ -453,8 +455,7 @@ li_package_read_component_data (LiPackage *pkg, const gchar *data, GError **erro
 	li_pkg_info_set_version (priv->info, version);
 
 	/* now get the package-id */
-	li_package_set_id (pkg,
-					li_pkg_info_get_id (priv->info));
+	li_package_set_id (pkg, li_pkg_info_get_id (priv->info));
 
 	return TRUE;
 }
@@ -706,14 +707,14 @@ li_package_install (LiPackage *pkg, GError **error)
 	struct archive_entry* en;
 	GError *tmp_error = NULL;
 	const gchar *pkg_id = NULL;
-	_cleanup_free_ gchar *pkg_root_dir = NULL;
+	g_autofree gchar *pkg_root_dir = NULL;
 	gchar *tmp;
 	gchar *tmp2;
 	gint res;
 	gboolean ret;
 	const gchar *version;
 	const gchar *tmp_payload_path;
-	_cleanup_object_unref_ LiExporter *exp = NULL;
+	g_autoptr(LiExporter) exp = NULL;
 	LiPackagePrivate *priv = GET_PRIVATE (pkg);
 
 	if (priv->remote_package) {
@@ -762,13 +763,16 @@ li_package_install (LiPackage *pkg, GError **error)
 		return FALSE;
 	}
 
-	if (priv->tlevel < LI_TRUST_LEVEL_LOW) {
-		/* we we have a below-low trust level, we either didn't validate yet or validation failed.
-		 * in both cases, better validate (again). */
-		li_package_verify_signature (pkg, &tmp_error);
-		if (tmp_error != NULL) {
-			g_propagate_error (error, tmp_error);
-			return FALSE;
+	/* verify the package signature before installing */
+	if (priv->auto_verify) {
+		if (priv->tlevel < LI_TRUST_LEVEL_LOW) {
+			/* we we have a below-low trust level, we either didn't validate yet or validation failed.
+			* in both cases, better validate (again). */
+			li_package_verify_signature (pkg, &tmp_error);
+			if (tmp_error != NULL) {
+				g_propagate_error (error, tmp_error);
+				return FALSE;
+			}
 		}
 	}
 
@@ -825,9 +829,9 @@ li_package_install (LiPackage *pkg, GError **error)
 	while (archive_read_next_header (payload_ar, &en) == ARCHIVE_OK) {
 		const gchar *filename;
 		gchar *path;
-		_cleanup_free_ gchar *tmp_str = NULL;
-		_cleanup_free_ gchar *dest_path = NULL;
-		_cleanup_free_ gchar *dest_fname = NULL;
+		g_autofree gchar *tmp_str = NULL;
+		g_autofree gchar *dest_path = NULL;
+		g_autofree gchar *dest_fname = NULL;
 
 		filename = archive_entry_pathname (en);
 		path = g_path_get_dirname (filename);
@@ -905,7 +909,7 @@ li_package_is_remote (LiPackage *pkg)
 gboolean
 li_package_download (LiPackage *pkg, GError **error)
 {
-	_cleanup_free_ gchar *pkg_fname = NULL;
+	g_autofree gchar *pkg_fname = NULL;
 	GError *tmp_error = NULL;
 	LiPackagePrivate *priv = GET_PRIVATE (pkg);
 
@@ -959,7 +963,7 @@ li_package_extract_embedded_package (LiPackage *pkg, LiPkgInfo *pki, GError **er
 	struct archive_entry* en;
 	gchar *fname;
 	gchar *hash;
-	_cleanup_free_ gchar *pkg_basename = NULL;
+	g_autofree gchar *pkg_basename = NULL;
 	LiPackage *subpkg;
 	GError *tmp_error = NULL;
 	LiPackagePrivate *priv = GET_PRIVATE (pkg);
@@ -1086,7 +1090,7 @@ li_package_extract_appstream_icons (LiPackage *pkg, const gchar *dest_dir, GErro
 	struct archive_entry* e;
 	gint res;
 	const gchar *tmp_payload_path;
-	_cleanup_free_ gchar *icon_dest_name = NULL;
+	g_autofree gchar *icon_dest_name = NULL;
 	GError *tmp_error = NULL;
 	LiPackagePrivate *priv = GET_PRIVATE (pkg);
 
@@ -1124,10 +1128,12 @@ li_package_extract_appstream_icons (LiPackage *pkg, const gchar *dest_dir, GErro
 		return;
 	}
 
-	icon_dest_name = g_strdup_printf ("%s.%s", priv->id, "png");
+	icon_dest_name = g_strdup_printf ("%s-%s.%s",
+					  li_pkg_info_get_name (priv->info),
+					  li_pkg_info_get_version (priv->info), "png");
 
 	while (archive_read_next_header (ar, &e) == ARCHIVE_OK) {
-		_cleanup_free_ gchar *tmp = NULL;
+		g_autofree gchar *tmp = NULL;
 		gchar *dest = NULL;
 
 		if (!g_str_has_suffix (archive_entry_pathname (e), ".png"))
@@ -1142,9 +1148,9 @@ li_package_extract_appstream_icons (LiPackage *pkg, const gchar *dest_dir, GErro
 		}
 
 		if (dest != NULL) {
-			_cleanup_free_ gchar *fname = NULL;
-			_cleanup_free_ gchar *src_fname = NULL;
-			_cleanup_free_ gchar *dest_fname = NULL;
+			g_autofree gchar *fname = NULL;
+			g_autofree gchar *src_fname = NULL;
+			g_autofree gchar *dest_fname = NULL;
 			g_mkdir_with_parents (dest, 0755);
 
 			fname = g_path_get_basename (archive_entry_pathname (e));
@@ -1184,7 +1190,7 @@ _li_package_signature_hash_matches (GHashTable *contents_hash, gchar **sigparts,
 {
 	gint i;
 	gboolean valid;
-	_cleanup_free_ gchar *hash = NULL;
+	g_autofree gchar *hash = NULL;
 
 	/* if we don't have that file which needs to be checked, we consider this as a match */
 	if (g_hash_table_lookup (contents_hash, fname) == NULL)
@@ -1220,7 +1226,7 @@ li_package_verify_signature (LiPackage *pkg, GError **error)
 {
 	GError *tmp_error = NULL;
 	const gchar *payload_fname;
-	_cleanup_free_ gchar *sig_content = NULL;
+	g_autofree gchar *sig_content = NULL;
 	LiTrustLevel level;
 	gchar **parts = NULL;
 	LiPackagePrivate *priv = GET_PRIVATE (pkg);
@@ -1332,7 +1338,7 @@ li_package_get_id (LiPackage *pkg)
  * li_package_set_id:
  * @unique_name: A unique name, build from the package name and version
  *
- * Se the unique name for this package.
+ * Set the unique name for this package.
  */
 void
 li_package_set_id (LiPackage *pkg, const gchar *unique_name)
@@ -1342,6 +1348,35 @@ li_package_set_id (LiPackage *pkg, const gchar *unique_name)
 	priv->id = g_strdup (unique_name);
 }
 
+/**
+ * li_package_get_auto_verify:
+ *
+ * %TRUE if the package should be automatically verified against the
+ * trusted keyring at install-time.
+ */
+gboolean
+li_package_get_auto_verify (LiPackage *pkg)
+{
+	LiPackagePrivate *priv = GET_PRIVATE (pkg);
+	return priv->auto_verify;
+}
+
+/**
+ * li_package_set_auto_verify:
+ * @verify: %TRUE if automatic verification should be done
+ *
+ * Set if the package should be automatically verified against the default
+ * keyring at install-time. Defaults to %TRUE.
+ * Do not disable this, unless you have already verified the package's origin
+ * (e.g. by checking the signature on the repository it originates from).
+ */
+void
+li_package_set_auto_verify (LiPackage *pkg, gboolean verify)
+{
+	LiPackagePrivate *priv = GET_PRIVATE (pkg);
+	priv->auto_verify = verify;
+}
+
 /**
  * li_package_get_info:
  *
