@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2014 Matthias Klumpp <matthias@tenstral.net>
  *
@@ -41,6 +41,7 @@
 #include "li-utils-private.h"
 #include "li-package.h"
 #include "li-pkg-index.h"
+#include "li-config-data.h"
 
 typedef struct _LiPkgBuilderPrivate	LiPkgBuilderPrivate;
 struct _LiPkgBuilderPrivate
@@ -97,7 +98,7 @@ li_get_package_fname (const gchar *root_dir, const gchar *disk_fname)
 {
 	gchar *tmp;
 	gchar *fname = NULL;
-	_cleanup_free_ gchar *root_prefix = NULL;
+	g_autofree gchar *root_prefix = NULL;
 	gint offset;
 
 	if (g_str_has_prefix (disk_fname, root_dir)) {
@@ -130,7 +131,7 @@ li_get_package_fname (const gchar *root_dir, const gchar *disk_fname)
  * li_pkg_builder_write_payload:
  */
 static void
-li_pkg_builder_write_payload (const gchar *input_dir, const gchar *out_fname)
+li_pkg_builder_write_payload (const gchar *input_dir, const gchar *out_fname, LiPackageKind kind, gboolean auto_filter)
 {
 	GPtrArray *files;
 	struct archive *a;
@@ -150,15 +151,23 @@ li_pkg_builder_write_payload (const gchar *input_dir, const gchar *out_fname)
 
 
 	for (i = 0; i < files->len; i++) {
-		gchar *ar_fname;
+		g_autofree gchar *ar_fname;
 		const gchar *fname = (const gchar *) g_ptr_array_index (files, i);
 
 		ar_fname = li_get_package_fname (input_dir, fname);
+		if (auto_filter) {
+			if (kind == LI_PACKAGE_KIND_DEVEL) {
+				if (!g_str_has_prefix (ar_fname, "include/"))
+					continue;
+			} else {
+				if (g_str_has_prefix (ar_fname, "include/"))
+					continue;
+			}
+		}
 
 		stat(fname, &st);
 		entry = archive_entry_new ();
 		archive_entry_set_pathname (entry, ar_fname);
-		g_free (ar_fname);
 
 		archive_entry_copy_stat (entry, &st);
 		archive_write_header (a, entry);
@@ -192,7 +201,7 @@ static gchar*
 li_pkg_builder_add_embedded_packages (const gchar *tmp_dir, const gchar *repo_source, GPtrArray *files, GError **error)
 {
 	guint i;
-	_cleanup_free_ gchar *pkgs_tmpdir = NULL;
+	g_autofree gchar *pkgs_tmpdir = NULL;
 	gchar *tmp;
 	GPtrArray *packages;
 	LiPkgIndex *idx;
@@ -344,7 +353,7 @@ li_print_gpgsign_result (gpgme_ctx_t ctx, gpgme_sign_result_t result, gpgme_sig_
 {
 	gpgme_invalid_key_t invkey;
 	gpgme_new_signature_t sig;
-	_cleanup_free_ gchar *short_fpr = NULL;
+	g_autofree gchar *short_fpr = NULL;
 	gpgme_key_t key;
 
 	for (invkey = result->invalid_signers; invkey; invkey = invkey->next)
@@ -365,14 +374,114 @@ li_print_gpgsign_result (gpgme_ctx_t ctx, gpgme_sign_result_t result, gpgme_sig_
 
 		if (gpgme_get_key (ctx, sig->fpr, &key, 0) == 0) {
 			if (key->uids != NULL)
-				g_print(_("Package signed for \"%s\" [0x%s]\n"), key->uids->uid, short_fpr);
+				g_debug ("Signed for \"%s\" [0x%s]", key->uids->uid, short_fpr);
 			gpgme_key_unref (key);
 		} else {
-			g_print(_("Package signed for 0x%s\n"), short_fpr);
+			g_debug ("Package signed for 0x%s", short_fpr);
 		}
 	}
 }
 
+/**
+ * li_pkg_builder_sign_data:
+ *
+ * Returns: Signed text, %NULL on error
+ */
+gpgme_data_t
+li_pkg_builder_sign_data (LiPkgBuilder *builder, const gchar *data, gpgme_sig_mode_t sigmode, GError **error)
+{
+	gpgme_error_t err;
+	gpgme_ctx_t ctx;
+	gpgme_data_t din, dout;
+	gpgme_sign_result_t sig_res;
+	LiPkgBuilderPrivate *priv = GET_PRIVATE (builder);
+
+	err = gpgme_new (&ctx);
+	if (err != 0) {
+		g_set_error (error,
+			LI_BUILDER_ERROR,
+			LI_BUILDER_ERROR_SIGN,
+			_("Signing of package failed: %s"),
+			gpgme_strsource (err));
+		gpgme_release (ctx);
+		return NULL;
+	}
+
+	gpgme_set_protocol (ctx, LI_GPG_PROTOCOL);
+	gpgme_set_armor (ctx, TRUE);
+
+	if (priv->gpg_key != NULL) {
+		gpgme_key_t akey;
+
+		err = gpgme_get_key (ctx, priv->gpg_key, &akey, 1);
+		if (err != 0) {
+			g_set_error (error,
+				LI_BUILDER_ERROR,
+				LI_BUILDER_ERROR_SIGN,
+				_("Signing of package failed: %s"),
+				gpgme_strsource (err));
+			gpgme_release (ctx);
+			return NULL;
+		}
+
+		err = gpgme_signers_add (ctx, akey);
+		if (err != 0) {
+			g_set_error (error,
+				LI_BUILDER_ERROR,
+				LI_BUILDER_ERROR_SIGN,
+				_("Signing of package failed: %s"),
+				gpgme_strsource (err));
+			gpgme_release (ctx);
+			return NULL;
+		}
+
+		gpgme_key_unref (akey);
+	}
+
+	err = gpgme_data_new_from_mem (&din, data, strlen (data), 0);
+	if (err != 0) {
+		g_set_error (error,
+			LI_BUILDER_ERROR,
+			LI_BUILDER_ERROR_SIGN,
+			_("Signing of package failed: %s"),
+			gpgme_strsource (err));
+		gpgme_data_release (din);
+		gpgme_release (ctx);
+		return NULL;
+	}
+
+	err = gpgme_data_new (&dout);
+	if (err != 0) {
+		g_set_error (error,
+			LI_BUILDER_ERROR,
+			LI_BUILDER_ERROR_SIGN,
+			_("Signing of package failed: %s"),
+			gpgme_strsource (err));
+		gpgme_data_release (din);
+		gpgme_release (ctx);
+		return NULL;
+	}
+
+	err = gpgme_op_sign (ctx, din, dout, sigmode);
+	sig_res = gpgme_op_sign_result (ctx);
+	if (sig_res)
+		li_print_gpgsign_result (ctx, sig_res, sigmode);
+
+	gpgme_data_release (din);
+	gpgme_release (ctx);
+
+	if (err != 0) {
+		g_set_error (error,
+			LI_BUILDER_ERROR,
+			LI_BUILDER_ERROR_SIGN,
+			_("Signing of package failed: %s"),
+			gpgme_strsource (err));
+		return NULL;
+	}
+
+	return dout;
+}
+
 /**
  * li_pkg_builder_sign_package:
  */
@@ -381,19 +490,15 @@ li_pkg_builder_sign_package (LiPkgBuilder *builder, const gchar *tmp_dir, GPtrAr
 {
 	guint i;
 	GString *index_str;
-	_cleanup_free_ gchar *indexdata = NULL;
-	gpgme_error_t err;
-	gpgme_ctx_t ctx;
-	gpgme_sig_mode_t sigmode = GPGME_SIG_MODE_NORMAL;
-	gpgme_data_t din, dout;
-	gpgme_sign_result_t sig_res;
+	g_autofree gchar *indexdata = NULL;
+	gpgme_data_t sigdata;
 
 	#define BUF_SIZE 512
 	gchar *sig_fname;
 	char buf[BUF_SIZE + 1];
 	FILE *file;
-	int ret;
-	LiPkgBuilderPrivate *priv = GET_PRIVATE (builder);
+	gint ret;
+	GError *tmp_error = NULL;
 
 	index_str = g_string_new ("");
 	for (i = 0; i < sign_files->len; i++) {
@@ -424,232 +529,247 @@ li_pkg_builder_sign_package (LiPkgBuilder *builder, const gchar *tmp_dir, GPtrAr
 	}
 	indexdata = g_string_free (index_str, FALSE);
 
-	err = gpgme_new (&ctx);
-	if (err != 0) {
-		g_set_error (error,
-				LI_BUILDER_ERROR,
-				LI_BUILDER_ERROR_SIGN,
-				_("Signing of package failed: %s"),
-				gpgme_strsource (err));
-		return NULL;
-	}
-
-	gpgme_set_protocol (ctx, LI_GPG_PROTOCOL);
-	gpgme_set_armor (ctx, TRUE);
-
-	if (priv->gpg_key != NULL) {
-		gpgme_key_t akey;
-
-		err = gpgme_get_key (ctx, priv->gpg_key, &akey, 1);
-		if (err != 0) {
-			g_set_error (error,
-					LI_BUILDER_ERROR,
-					LI_BUILDER_ERROR_SIGN,
-					_("Signing of package failed: %s"),
-					gpgme_strsource (err));
-			return NULL;
-		}
-
-		err = gpgme_signers_add (ctx, akey);
-		if (err != 0) {
-			g_set_error (error,
-					LI_BUILDER_ERROR,
-					LI_BUILDER_ERROR_SIGN,
-					_("Signing of package failed: %s"),
-					gpgme_strsource (err));
-			return NULL;
-		}
-
-		gpgme_key_unref (akey);
-	}
-
-	err = gpgme_data_new_from_mem (&din, indexdata, strlen (indexdata), 0);
-	if (err != 0) {
-		g_set_error (error,
-				LI_BUILDER_ERROR,
-				LI_BUILDER_ERROR_SIGN,
-				_("Signing of package failed: %s"),
-				gpgme_strsource (err));
-		return NULL;
-	}
-
-	err = gpgme_data_new (&dout);
-	if (err != 0) {
-		g_set_error (error,
-				LI_BUILDER_ERROR,
-				LI_BUILDER_ERROR_SIGN,
-				_("Signing of package failed: %s"),
-				gpgme_strsource (err));
-		return NULL;
-	}
-
-	err = gpgme_op_sign (ctx, din, dout, sigmode);
-	sig_res = gpgme_op_sign_result (ctx);
-	if (sig_res)
-		li_print_gpgsign_result (ctx, sig_res, sigmode);
-
-	if (err != 0) {
-		g_set_error (error,
-				LI_BUILDER_ERROR,
-				LI_BUILDER_ERROR_SIGN,
-				_("Signing of package failed: %s"),
-				gpgme_strsource (err));
+	sigdata = li_pkg_builder_sign_data (builder, indexdata, GPGME_SIG_MODE_NORMAL, &tmp_error);
+	if (tmp_error != NULL) {
+		g_propagate_error (error, tmp_error);
 		return NULL;
 	}
+	g_print ("%s\n", _("Package signed."));
 
 	sig_fname = g_build_filename (tmp_dir, "_signature", NULL);
 	file = fopen (sig_fname, "w");
 	if (file == NULL) {
 		g_set_error (error,
-				LI_BUILDER_ERROR,
-				LI_BUILDER_ERROR_SIGN,
-				_("Unable to write signature: %s"),
-				g_strerror (errno));
+			LI_BUILDER_ERROR,
+			LI_BUILDER_ERROR_SIGN,
+			_("Unable to write signature: %s"),
+			g_strerror (errno));
 		g_free (sig_fname);
+		gpgme_data_release (sigdata);
 		return NULL;
 	}
 
-	gpgme_data_seek (dout, 0, SEEK_SET);
-	while ((ret = gpgme_data_read (dout, buf, BUF_SIZE)) > 0)
+	gpgme_data_seek (sigdata, 0, SEEK_SET);
+	while ((ret = gpgme_data_read (sigdata, buf, BUF_SIZE)) > 0)
 		fwrite (buf, ret, 1, file);
 	fclose (file);
 
-	gpgme_data_release (dout);
-	gpgme_data_release (din);
-	gpgme_release (ctx);
+	gpgme_data_release (sigdata);
 
 	return sig_fname;
 }
 
 /**
- * li_pkg_builder_create_package_from_dir:
+ * li_pkg_builder_filelist_entry_for_pkgfile:
  */
-gboolean
-li_pkg_builder_create_package_from_dir (LiPkgBuilder *builder, const gchar *dir, const gchar *out_fname, GError **error)
+static gchar*
+li_pkg_builder_filelist_entry_for_pkgfile (LiPkgBuilder *builder, const gchar *pkg_fname)
 {
-	_cleanup_free_ gchar *ctl_fname = NULL;
-	_cleanup_free_ gchar *payload_root = NULL;
-	_cleanup_free_ gchar *repo_root = NULL;
-	_cleanup_free_ gchar *as_metadata = NULL;
-	_cleanup_free_ gchar *tmp_dir = NULL;
-	_cleanup_free_ gchar *payload_file = NULL;
-	_cleanup_free_ gchar *pkg_fname = NULL;
-	_cleanup_free_ gchar *sig_fname = NULL;
-	_cleanup_object_unref_ GFile *ctlfile = NULL;
-	_cleanup_object_unref_ LiPkgInfo *ctl = NULL;
-	_cleanup_ptrarray_unref_ GPtrArray *files = NULL;
-	_cleanup_ptrarray_unref_ GPtrArray *sign_files = NULL;
-	GError *tmp_error = NULL;
 	gchar *tmp;
+	gchar *tmp2;
+	gchar *tmp3;
+
+	if (!g_file_test (pkg_fname, G_FILE_TEST_EXISTS))
+		return NULL;
+
+	tmp2 = g_path_get_basename (pkg_fname);
+	tmp3 = li_compute_checksum_for_file (pkg_fname);
+	tmp = g_strdup_printf ("%s %s", tmp3, tmp2);
+	g_free (tmp2);
+	g_free (tmp3);
+
+	return tmp;
+}
+
+/**
+ * li_pkg_builder_write_dsc_file:
+ *
+ * The .dsc description is meant to be used to upload one or more packages
+ * to a remote repository.
+ * This function generates a basic .dsc file for the new package.
+ */
+void
+li_pkg_builder_write_dsc_file (LiPkgBuilder *builder, const gchar *pkg_fname_rt, const gchar *pkg_fname_sdk, LiPkgInfo *ctl, GError **error)
+{
+	g_autoptr(LiConfigData) cdata = NULL;
+	g_autofree gchar *fname = NULL;
+	g_autofree gchar *email = NULL;
+	g_autofree gchar *username = NULL;
+	g_autofree gchar *target_repo = NULL;
+	gchar *tmp;
+	gchar *tmp2;
+	gchar *tmp3;
+
+	gpgme_data_t sigdata;
+	#define BUF_SIZE 512
+	char buf[BUF_SIZE + 1];
+	FILE *file;
+	gint ret;
+	GError *tmp_error = NULL;
+
+	cdata = li_config_data_new ();
+
+	/* set basic information */
+	li_config_data_set_value (cdata, "Limba-Version", VERSION);
+
+	tmp2 = li_pkg_builder_filelist_entry_for_pkgfile (builder, pkg_fname_rt);
+	tmp3 = li_pkg_builder_filelist_entry_for_pkgfile (builder, pkg_fname_sdk);
+	if (tmp3 == NULL)
+		tmp = g_strdup (tmp2);
+	else
+		tmp = g_strdup_printf ("%s\n%s", tmp2, tmp3);
+	li_config_data_set_value (cdata, "Files", tmp);
+	g_free (tmp);
+	g_free (tmp2);
+	g_free (tmp3);
+	tmp = NULL;
+
+	/* set uploader field */
+	email = li_env_get_user_email ();
+	username = li_env_get_user_fullname ();
+	target_repo = li_env_get_target_repo ();
+
+	if (username == NULL) {
+		if (email != NULL)
+			tmp = g_strdup (email);
+	} else {
+		if (email != NULL)
+			tmp = g_strdup_printf ("%s <%s>", username, email);
+	}
+	if (tmp != NULL) {
+		li_config_data_set_value (cdata, "Uploader", tmp);
+		g_free (tmp);
+	}
+
+	/* set target repository */
+	li_config_data_set_value (cdata, "Target", target_repo);
+
+	tmp = li_config_data_get_data (cdata);
+	sigdata = li_pkg_builder_sign_data (builder, tmp, GPGME_SIG_MODE_CLEAR, &tmp_error);
+	g_free (tmp);
+	if (tmp_error != NULL) {
+		g_propagate_error (error, tmp_error);
+		return;
+	}
+	g_print ("%s\n", _("DSC file signed."));
+
+	fname = g_strdup_printf ("%s.dsc", pkg_fname_rt);
+	tmp = li_config_data_get_data (cdata);
+
+	file = fopen (fname, "w");
+	if (file == NULL) {
+		g_set_error (error,
+			LI_BUILDER_ERROR,
+			LI_BUILDER_ERROR_SIGN,
+			_("Unable to write signature on dsc file: %s"),
+			g_strerror (errno));
+		gpgme_data_release (sigdata);
+		return;
+	}
+
+	gpgme_data_seek (sigdata, 0, SEEK_SET);
+	while ((ret = gpgme_data_read (sigdata, buf, BUF_SIZE)) > 0)
+		fwrite (buf, ret, 1, file);
+	fclose (file);
+
+	gpgme_data_release (sigdata);
+}
+
+/**
+ * li_pkg_builder_build_package_with_details:
+ *
+ * Helper function, to build a package with given parameters.
+ * This function expects all its parameters to be valid!
+ */
+static gboolean
+li_pkg_builder_build_package_with_details (LiPkgBuilder *builder, LiPkgInfo *ctl, LiPackageKind kind, AsComponent *cpt, const gchar *payload_root, const gchar *pkg_fname, gboolean split_sdk, GError **error)
+{
+	g_autofree gchar *tmp_dir = NULL;
+	g_autofree gchar *ctl_fname = NULL;
+	g_autofree gchar *asdata_fname = NULL;
+
+	g_autofree gchar *payload_file = NULL;
+	g_autofree gchar *sig_fname = NULL;
+	g_autoptr(GPtrArray) files = NULL;
+	g_autoptr(GPtrArray) sign_files = NULL;
+	g_autoptr (AsMetadata) metad = NULL;
+
+	g_autofree gchar *cpt_orig_id = NULL;
+	g_autofree gchar *orig_deps = NULL;
+	g_autofree gchar *orig_sdk_deps = NULL;
+
+	GError *tmp_error = NULL;
 	LiPkgBuilderPrivate *priv = GET_PRIVATE (builder);
 
-	tmp = g_build_filename (dir, "control", NULL);
-	ctlfile = g_file_new_for_path (tmp);
-	g_free (tmp);
-	if (!g_file_query_exists (ctlfile, NULL)) {
-		g_set_error (error,
-				LI_BUILDER_ERROR,
-				LI_BUILDER_ERROR_NOT_FOUND,
-				_("Could not find control file for the archive!"));
-		return FALSE;
+	tmp_dir = li_utils_get_tmp_dir ("build");
+	payload_file = g_build_filename (tmp_dir, "main-data.tar.xz", NULL);
+
+	if ((split_sdk) && (kind == LI_PACKAGE_KIND_DEVEL)) {
+		g_autofree gchar *tmp = NULL;
+
+		tmp = g_build_filename (payload_root, "include", NULL);
+		if (!g_file_test (tmp, G_FILE_TEST_IS_DIR)) {
+			g_free (tmp);
+			tmp = g_build_filename (payload_root, "opt", "bundle", "include", NULL);
+			if (!g_file_test (tmp, G_FILE_TEST_IS_DIR)) {
+				/* no headers? No automatically built SDK package! */
+				return TRUE;
+			}
+		}
 	}
-	ctl = li_pkg_info_new ();
-	li_pkg_info_load_file (ctl, ctlfile, &tmp_error);
+
+	/* create payload */
+	li_pkg_builder_write_payload (payload_root, payload_file, kind, split_sdk);
+
+	/* prepare component metadata */
+	metad = as_metadata_new ();
+	as_metadata_set_locale (metad, "ALL");
+	asdata_fname = g_build_filename (tmp_dir, "metainfo.xml", NULL);
+	cpt_orig_id = g_strdup (as_component_get_id (cpt));
+
+	/* development components must have an ID in the form of <cptid>.sdk, e.g. io.qt.Qt5Core.sdk */
+	if (kind == LI_PACKAGE_KIND_DEVEL) {
+		g_autofree gchar *sdk_id = NULL;
+
+		sdk_id = g_strdup_printf ("%s.sdk", cpt_orig_id);
+		as_component_set_id (cpt, sdk_id);
+	}
+
+	as_metadata_add_component (metad, cpt);
+	as_metadata_save_upstream_xml (metad, asdata_fname, &tmp_error);
+	as_component_set_id (cpt, cpt_orig_id);
 	if (tmp_error != NULL) {
 		g_propagate_error (error, tmp_error);
 		return FALSE;
 	}
 
-	payload_root = g_build_filename (dir, "inst_target", NULL);
-	if (!g_file_test (payload_root, G_FILE_TEST_IS_DIR)) {
-		g_set_error (error,
-				LI_BUILDER_ERROR,
-				LI_BUILDER_ERROR_NOT_FOUND,
-				_("Could not find payload data in the 'inst_target' subdirectory."));
-		return FALSE;
-	}
-
-	as_metadata = g_build_filename (dir, "metainfo.xml", NULL);
-	if (!g_file_test (as_metadata, G_FILE_TEST_IS_REGULAR)) {
-		g_set_error (error,
-				LI_BUILDER_ERROR,
-				LI_BUILDER_ERROR_NOT_FOUND,
-				_("Could not find AppStream metadata for the new package!"));
-		return FALSE;
-	}
-
-	repo_root = g_build_filename (dir, "repo", NULL);
-	if (!g_file_test (repo_root, G_FILE_TEST_IS_DIR)) {
-		/* we have no dependency repository */
-		g_free (repo_root);
-		repo_root = NULL;
-	}
-
-	if (out_fname == NULL) {
-		AsMetadata *mdata;
-		AsComponent *cpt;
-		GFile *asfile;
-		gchar *tmp;
-		const gchar *version;
-
-		/* we need to auto-generate a package filename */
-		asfile = g_file_new_for_path (as_metadata);
-		if (!g_file_query_exists (asfile, NULL)) {
-			g_set_error (error,
-				LI_BUILDER_ERROR,
-				LI_BUILDER_ERROR_FAILED,
-				_("Could not generate package filename: AppStream metadata is missing."));
-			g_object_unref (asfile);
-			return FALSE;
-		}
-
-		mdata = as_metadata_new ();
-		as_metadata_set_locale (mdata, "ALL");
-
-		as_metadata_parse_file (mdata, asfile, &tmp_error);
-		cpt = as_metadata_get_component (mdata);
-		g_object_unref (asfile);
-		if (tmp_error != NULL) {
-			g_propagate_error (error, tmp_error);
-			g_object_unref (mdata);
-			return FALSE;
-		}
-
-		tmp = li_str_replace (as_component_get_name (cpt), " ", "");
-		version = li_get_last_version_from_component (cpt);
-		if (version != NULL)
-			pkg_fname = g_strdup_printf ("%s/%s-%s.ipk", dir, tmp, version);
-		else
-			pkg_fname = g_strdup_printf ("%s/%s.ipk", dir, tmp);
-		g_free (tmp);
-		g_object_unref (mdata);
-	} else {
-		pkg_fname = g_strdup (out_fname);
-	}
-
-	tmp_dir = li_utils_get_tmp_dir ("build");
-	payload_file = g_build_filename (tmp_dir, "main-data.tar.xz", NULL);
-
-	/* create payload */
-	li_pkg_builder_write_payload (payload_root, payload_file);
-
 	/* construct package contents */
 	files = g_ptr_array_new_with_free_func (g_free);
 	sign_files = g_ptr_array_new ();
 
-	if (repo_root != NULL) {
-		gchar *repo_fname;
-		/* we have extra packages to embed */
-		repo_fname = li_pkg_builder_add_embedded_packages (tmp_dir, repo_root, files, &tmp_error);
-		if (tmp_error != NULL) {
-			g_propagate_error (error, tmp_error);
-			return FALSE;
+	/* check if we should embed a repository in the package structure */
+	if (kind == LI_PACKAGE_KIND_COMMON) {
+		g_autofree gchar *repo_root = NULL;
+
+		repo_root = g_build_filename (payload_root, "..", "repo", NULL);
+		if (g_file_test (repo_root, G_FILE_TEST_IS_DIR)) {
+			gchar *repo_fname;
+
+			/* we have a dependency repo, embed extra packages */
+			repo_fname = li_pkg_builder_add_embedded_packages (tmp_dir, repo_root, files, &tmp_error);
+			if (tmp_error != NULL) {
+				g_propagate_error (error, tmp_error);
+				return FALSE;
+			}
+			g_ptr_array_add (files, repo_fname);
+			/* we need to sign the repo file */
+			g_ptr_array_add (sign_files, repo_fname);
 		}
-		g_ptr_array_add (files, repo_fname);
-		/* we need to sign the repo file */
-		g_ptr_array_add (sign_files, repo_fname);
 	}
 
+	/* set the package type we're building here */
+	li_pkg_info_set_kind (ctl, kind);
+
 	/* handle arch:any notation (resolve to current arch) */
 	if (g_strcmp0 (li_pkg_info_get_architecture (ctl), "any") == 0) {
 		gchar *arch;
@@ -658,18 +778,42 @@ li_pkg_builder_create_package_from_dir (LiPkgBuilder *builder, const gchar *dir,
 		g_free (arch);
 	}
 
-	/* safe our new control metadata */
+	/* Ensure SDK packages always depend on their runtime package */
+	orig_deps = g_strdup (li_pkg_info_get_dependencies (ctl));
+	orig_sdk_deps = g_strdup (li_pkg_info_get_sdk_dependencies (ctl));
+	if (kind == LI_PACKAGE_KIND_DEVEL) {
+		g_autofree gchar *tmp = NULL;
+
+		if ((li_pkg_info_get_sdk_dependencies (ctl) == NULL) || (g_strcmp0 (li_pkg_info_get_sdk_dependencies (ctl), "") == 0)) {
+			tmp = g_strdup_printf ("%s (== %s)",
+						cpt_orig_id,
+						li_get_last_version_from_component (cpt));
+		} else {
+			tmp = g_strdup_printf ("%s (== %s), %s",
+						cpt_orig_id,
+						li_get_last_version_from_component (cpt),
+						li_pkg_info_get_sdk_dependencies (ctl));
+		}
+		li_pkg_info_set_dependencies (ctl, tmp);
+	}
+
+	/* save our new control metadata */
+	li_pkg_info_set_sdk_dependencies (ctl, NULL);
 	ctl_fname = g_build_filename (tmp_dir, "control", NULL);
 	li_pkg_info_save_to_file (ctl, ctl_fname);
 
+	/* restore LiPkgInfo to previous state */
+	li_pkg_info_set_dependencies (ctl, orig_deps);
+	li_pkg_info_set_sdk_dependencies (ctl, orig_sdk_deps);
+
 	/* we want these files in the package */
 	g_ptr_array_add (files, g_strdup (ctl_fname));
-	g_ptr_array_add (files, g_strdup (as_metadata));
+	g_ptr_array_add (files, g_strdup (asdata_fname));
 	g_ptr_array_add (files, g_strdup (payload_file));
 
 	/* these files need to be signed in order to verify the whole package */
 	g_ptr_array_add (sign_files, ctl_fname);
-	g_ptr_array_add (sign_files, as_metadata);
+	g_ptr_array_add (sign_files, asdata_fname);
 	g_ptr_array_add (sign_files, payload_file);
 
 	if (priv->sign_package) {
@@ -696,6 +840,160 @@ li_pkg_builder_create_package_from_dir (LiPkgBuilder *builder, const gchar *dir,
 	return TRUE;
 }
 
+/**
+ * li_pkg_builder_create_package_from_dir:
+ */
+gboolean
+li_pkg_builder_create_package_from_dir (LiPkgBuilder *builder, const gchar *dir, const gchar *out_fname, GError **error)
+{
+	g_autofree gchar *payload_root_rt = NULL;
+	g_autofree gchar *payload_root_sdk = NULL;
+	g_autofree gchar *asdata_fname = NULL;
+
+	g_autofree gchar *pkg_fname_rt = NULL;
+	g_autofree gchar *pkg_fname_sdk = NULL;
+	g_autoptr(GFile) ctlfile = NULL;
+	g_autoptr(LiPkgInfo) ctl = NULL;
+	g_autoptr(AsComponent) cpt = NULL;
+	g_autoptr(AsMetadata) mdata = NULL;
+	g_autoptr(GFile) asfile = NULL;
+	gboolean auto_sdkpkg;
+	GError *tmp_error = NULL;
+	gchar *ctlfile_fname;
+
+	ctlfile_fname = g_build_filename (dir, "control", NULL);
+	ctlfile = g_file_new_for_path (ctlfile_fname);
+	g_free (ctlfile_fname);
+	if (!g_file_query_exists (ctlfile, NULL)) {
+		g_set_error (error,
+				LI_BUILDER_ERROR,
+				LI_BUILDER_ERROR_NOT_FOUND,
+				_("Could not find control file for the archive!"));
+		return FALSE;
+	}
+	ctl = li_pkg_info_new ();
+	li_pkg_info_load_file (ctl, ctlfile, &tmp_error);
+	if (tmp_error != NULL) {
+		g_propagate_error (error, tmp_error);
+		return FALSE;
+	}
+
+	/* load AppStream metadata */
+	asdata_fname = g_build_filename (dir, "metainfo.xml", NULL);
+	asfile = g_file_new_for_path (asdata_fname);
+	if (!g_file_query_exists (asfile, NULL)) {
+		g_set_error (error,
+			LI_BUILDER_ERROR,
+			LI_BUILDER_ERROR_FAILED,
+			_("Could not build package: AppStream metadata is missing."));
+		return FALSE;
+	}
+
+	mdata = as_metadata_new ();
+	as_metadata_set_locale (mdata, "ALL");
+
+	as_metadata_parse_file (mdata, asfile, &tmp_error);
+	if (tmp_error != NULL) {
+		g_propagate_error (error, tmp_error);
+		return FALSE;
+	}
+	cpt = as_metadata_get_component (mdata);
+	g_object_ref (cpt);
+
+	if (out_fname == NULL) {
+		gchar *tmp;
+		const gchar *version;
+
+		/* we need to auto-generate a package filename */
+		tmp = li_str_replace (as_component_get_name (cpt), " ", "");
+		version = li_get_last_version_from_component (cpt);
+		if (version != NULL) {
+			pkg_fname_rt  = g_strdup_printf ("%s/%s-%s.ipk", dir, tmp, version);
+			pkg_fname_sdk = g_strdup_printf ("%s/%s-%s.devel.ipk", dir, tmp, version);
+		} else {
+			pkg_fname_rt  = g_strdup_printf ("%s/%s.ipk", dir, tmp);
+			pkg_fname_sdk = g_strdup_printf ("%s/%s.devel.ipk", dir, tmp);
+		}
+		g_free (tmp);
+	} else {
+		g_autofree gchar *base_fname;
+		g_autofree gchar *dirname;
+
+		pkg_fname_rt = g_strdup (out_fname);
+
+		base_fname = g_path_get_basename (out_fname);
+		dirname = g_path_get_dirname (out_fname);
+		pkg_fname_sdk = g_strdup_printf ("%s/devel-%s", dirname, base_fname);
+	}
+
+	/* search for runtime payload */
+	auto_sdkpkg = TRUE;
+	payload_root_rt = g_build_filename (dir, "target", NULL);
+	if (!g_file_test (payload_root_rt, G_FILE_TEST_IS_DIR)) {
+		/* we don't automatically build an SDK package anymore */
+		auto_sdkpkg = FALSE;
+
+		g_free (payload_root_rt);
+		payload_root_rt = g_build_filename (dir, "rt.target", NULL);
+		if (!g_file_test (payload_root_rt, G_FILE_TEST_IS_DIR)) {
+			g_set_error (error,
+				LI_BUILDER_ERROR,
+				LI_BUILDER_ERROR_NOT_FOUND,
+				_("Could not find payload data in the 'target' or 'rt.target' subdirectory."));
+			return FALSE;
+		}
+	}
+
+	/* search if we have a dedicated SDK payload */
+	payload_root_sdk = g_build_filename (dir, "sdk.target", NULL);
+	if (!g_file_test (payload_root_sdk, G_FILE_TEST_IS_DIR)) {
+		g_free (payload_root_sdk);
+		payload_root_sdk = NULL;
+	}
+
+	if (auto_sdkpkg) {
+		/* build package, automatically detect if we need a development package */
+		li_pkg_builder_build_package_with_details (builder, ctl, LI_PACKAGE_KIND_COMMON, cpt, payload_root_rt, pkg_fname_rt, TRUE, &tmp_error);
+		if (tmp_error != NULL) {
+			g_propagate_error (error, tmp_error);
+			return FALSE;
+		}
+
+		/* run the development package build (will return no package if no SDK components are auto-detected) */
+		li_pkg_builder_build_package_with_details (builder, ctl, LI_PACKAGE_KIND_DEVEL, cpt, payload_root_rt, pkg_fname_sdk, TRUE, &tmp_error);
+		if (tmp_error != NULL) {
+			g_propagate_error (error, tmp_error);
+			return FALSE;
+		}
+
+	} else {
+		/* build RT package */
+		li_pkg_builder_build_package_with_details (builder, ctl, LI_PACKAGE_KIND_COMMON, cpt, payload_root_rt, pkg_fname_rt, FALSE, &tmp_error);
+		if (tmp_error != NULL) {
+			g_propagate_error (error, tmp_error);
+			return FALSE;
+		}
+
+		if (payload_root_sdk != NULL) {
+			/* build SDK package */
+			li_pkg_builder_build_package_with_details (builder, ctl, LI_PACKAGE_KIND_DEVEL, cpt, payload_root_sdk, pkg_fname_sdk, FALSE, &tmp_error);
+			if (tmp_error != NULL) {
+				g_propagate_error (error, tmp_error);
+				return FALSE;
+			}
+		}
+	}
+
+	/* write dsc file */
+	li_pkg_builder_write_dsc_file (builder, pkg_fname_rt, pkg_fname_sdk, ctl, &tmp_error);
+	if (tmp_error != NULL) {
+		g_propagate_error (error, tmp_error);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 /**
  * li_pkg_builder_get_sign_package:
  */
