@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2015 Matthias Klumpp <matthias@tenstral.net>
  *
@@ -43,11 +43,12 @@
 #include "li-pkg-index.h"
 #include "li-package.h"
 #include "li-package-private.h"
+#include "li-repo-entry.h"
 
 typedef struct _LiRepositoryPrivate	LiRepositoryPrivate;
 struct _LiRepositoryPrivate
 {
-	GHashTable *indices;
+	GHashTable *indices; /* of string -> GHashTable */
 	GHashTable *asmeta;
 	gchar *repo_path;
 
@@ -83,7 +84,7 @@ li_repository_init (LiRepository *repo)
 {
 	LiRepositoryPrivate *priv = GET_PRIVATE (repo);
 
-	priv->indices = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
+	priv->indices = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_hash_table_unref);
 	priv->asmeta = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 	priv->rconfig = li_config_data_new ();
 }
@@ -92,15 +93,25 @@ li_repository_init (LiRepository *repo)
  * li_repository_get_index:
  */
 static LiPkgIndex*
-li_repository_get_index (LiRepository *repo, const gchar *arch)
+li_repository_get_index (LiRepository *repo, LiRepoIndexKinds kind, const gchar *arch)
 {
 	LiPkgIndex *index;
+	GHashTable *index_tab;
 	LiRepositoryPrivate *priv = GET_PRIVATE (repo);
 
-	index = g_hash_table_lookup (priv->indices, arch);
+	index_tab = g_hash_table_lookup (priv->indices,
+					li_repo_index_kind_to_string (kind));
+	if (index_tab == NULL) {
+		index_tab = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
+		g_hash_table_insert (priv->indices,
+					g_strdup (li_repo_index_kind_to_string (kind)),
+					index_tab);
+	}
+
+	index = g_hash_table_lookup (index_tab, arch);
 	if (index == NULL) {
 		index = li_pkg_index_new ();
-		g_hash_table_insert (priv->indices, g_strdup (arch), index);
+		g_hash_table_insert (index_tab, g_strdup (arch), index);
 	}
 
 	return index;
@@ -127,6 +138,41 @@ li_repository_get_asmeta (LiRepository *repo, const gchar *arch)
 	return metad;
 }
 
+/**
+ * li_repository_load_index:
+ */
+static void
+li_repository_load_index (LiRepository *repo, const gchar *path, LiRepoIndexKinds kind, const gchar *arch, GError **error)
+{
+	g_autofree gchar *fname = NULL;
+	g_autoptr(GFile) file = NULL;
+	GError *tmp_error = NULL;
+
+	if (kind == LI_REPO_INDEX_KIND_NONE)
+		return;
+
+	if (kind == LI_REPO_INDEX_KIND_COMMON)
+		fname = g_build_filename (path, "Index.gz", NULL);
+	else if (kind == LI_REPO_INDEX_KIND_DEVEL)
+		fname = g_build_filename (path, "Index-Devel.gz", NULL);
+	else if (kind == LI_REPO_INDEX_KIND_SOURCE)
+		fname = g_build_filename (path, "Index-Sources.gz", NULL);
+	else
+		g_critical ("Handling unknown repository index type: %i", (gint) kind);
+
+	file = g_file_new_for_path (fname);
+	if (g_file_query_exists (file, NULL)) {
+		LiPkgIndex *index;
+
+		index = li_repository_get_index (repo, kind, arch);
+		li_pkg_index_load_file (index, file, &tmp_error);
+	}
+	if (tmp_error != NULL) {
+		g_propagate_error (error, tmp_error);
+		return;
+	}
+}
+
 /**
  * li_repository_load_indices:
  */
@@ -150,7 +196,7 @@ li_repository_load_indices (LiRepository *repo, const gchar* dir, GError **error
 	}
 
 	while ((file_info = g_file_enumerator_next_file (enumerator, NULL, &tmp_error)) != NULL) {
-		_cleanup_free_ gchar *path = NULL;
+		g_autofree gchar *path = NULL;
 
 		if (tmp_error != NULL) {
 			g_propagate_error (error, tmp_error);
@@ -160,34 +206,28 @@ li_repository_load_indices (LiRepository *repo, const gchar* dir, GError **error
 		if (g_file_info_get_is_hidden (file_info))
 			continue;
 		path = g_build_filename (dir,
-								g_file_info_get_name (file_info),
-								NULL);
+					 g_file_info_get_name (file_info),
+					 NULL);
 
 		if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
-			_cleanup_free_ gchar *arch = NULL;
-			_cleanup_free_ gchar *fname = NULL;
+			g_autofree gchar *arch = NULL;
+			g_autofree gchar *fname = NULL;
 			GFile *file;
 
 			/* our directory name is the architecture */
 			arch = g_path_get_basename (path);
 
-			/* load IPK package index */
-			fname = g_build_filename (path, "Index.gz", NULL);
-			file = g_file_new_for_path (fname);
-			if (g_file_query_exists (file, NULL)) {
-				LiPkgIndex *index;
-
-				index = li_pkg_index_new ();
-				li_pkg_index_load_file (index, file, &tmp_error);
-				if (tmp_error == NULL)
-					g_hash_table_insert (priv->indices, g_strdup (arch), index);
+			/* load IPK package indices  */
+			li_repository_load_index (repo, path, LI_REPO_INDEX_KIND_COMMON, arch, &tmp_error);
+			if (tmp_error != NULL) {
+				g_propagate_error (error, tmp_error);
+				goto out;
 			}
-			g_object_unref (file);
+			li_repository_load_index (repo, path, LI_REPO_INDEX_KIND_DEVEL, arch, &tmp_error);
 			if (tmp_error != NULL) {
 				g_propagate_error (error, tmp_error);
 				goto out;
 			}
-			g_free (fname);
 
 			/* load AppStream metadata (if present) */
 			fname = g_build_filename (path, "Metadata.xml.gz", NULL);
@@ -287,16 +327,16 @@ li_repository_sign (LiRepository *repo, const gchar *sigtext, GError **error)
 	char buf[BUF_SIZE + 1];
 	FILE *file;
 	int ret;
-	_cleanup_free_ gchar *gpg_key = NULL;
+	g_autofree gchar *gpg_key = NULL;
 	LiRepositoryPrivate *priv = GET_PRIVATE (repo);
 
 	err = gpgme_new (&ctx);
 	if (err != 0) {
 		g_set_error (error,
-				LI_REPOSITORY_ERROR,
-				LI_REPOSITORY_ERROR_SIGN,
-				_("Signing of repository failed: %s"),
-				gpgme_strsource (err));
+			LI_REPOSITORY_ERROR,
+			LI_REPOSITORY_ERROR_SIGN,
+			_("Signing of repository failed: %s"),
+			gpgme_strsource (err));
 		return;
 	}
 
@@ -311,20 +351,20 @@ li_repository_sign (LiRepository *repo, const gchar *sigtext, GError **error)
 		err = gpgme_get_key (ctx, gpg_key, &akey, 1);
 		if (err != 0) {
 			g_set_error (error,
-					LI_REPOSITORY_ERROR,
-					LI_REPOSITORY_ERROR_SIGN,
-					_("Signing of repository failed: %s"),
-					gpgme_strsource (err));
+				LI_REPOSITORY_ERROR,
+				LI_REPOSITORY_ERROR_SIGN,
+				_("Signing of repository failed: %s"),
+				gpgme_strsource (err));
 			return;
 		}
 
 		err = gpgme_signers_add (ctx, akey);
 		if (err != 0) {
 			g_set_error (error,
-					LI_REPOSITORY_ERROR,
-					LI_REPOSITORY_ERROR_SIGN,
-					_("Signing of repository failed: %s"),
-					gpgme_strsource (err));
+				LI_REPOSITORY_ERROR,
+				LI_REPOSITORY_ERROR_SIGN,
+				_("Signing of repository failed: %s"),
+				gpgme_strsource (err));
 			return;
 		}
 
@@ -334,20 +374,20 @@ li_repository_sign (LiRepository *repo, const gchar *sigtext, GError **error)
 	err = gpgme_data_new_from_mem (&din, sigtext, strlen (sigtext), 0);
 	if (err != 0) {
 		g_set_error (error,
-				LI_REPOSITORY_ERROR,
-				LI_REPOSITORY_ERROR_SIGN,
-				_("Signing of repository failed: %s"),
-				gpgme_strsource (err));
+			LI_REPOSITORY_ERROR,
+			LI_REPOSITORY_ERROR_SIGN,
+			_("Signing of repository failed: %s"),
+			gpgme_strsource (err));
 		return;
 	}
 
 	err = gpgme_data_new (&dout);
 	if (err != 0) {
 		g_set_error (error,
-				LI_REPOSITORY_ERROR,
-				LI_REPOSITORY_ERROR_SIGN,
-				_("Signing of repository failed: %s"),
-				gpgme_strsource (err));
+			LI_REPOSITORY_ERROR,
+			LI_REPOSITORY_ERROR_SIGN,
+			_("Signing of repository failed: %s"),
+			gpgme_strsource (err));
 		return;
 	}
 
@@ -355,10 +395,10 @@ li_repository_sign (LiRepository *repo, const gchar *sigtext, GError **error)
 
 	if (err != 0) {
 		g_set_error (error,
-				LI_REPOSITORY_ERROR,
-				LI_REPOSITORY_ERROR_SIGN,
-				_("Signing of repository failed: %s"),
-				gpgme_strsource (err));
+			LI_REPOSITORY_ERROR,
+			LI_REPOSITORY_ERROR_SIGN,
+			_("Signing of repository failed: %s"),
+			gpgme_strsource (err));
 		return;
 	}
 
@@ -366,10 +406,10 @@ li_repository_sign (LiRepository *repo, const gchar *sigtext, GError **error)
 	file = fopen (sig_fname, "w");
 	if (file == NULL) {
 		g_set_error (error,
-				LI_REPOSITORY_ERROR,
-				LI_REPOSITORY_ERROR_SIGN,
-				_("Unable to write signature: %s"),
-				g_strerror (errno));
+			LI_REPOSITORY_ERROR,
+			LI_REPOSITORY_ERROR_SIGN,
+			_("Unable to write signature: %s"),
+			g_strerror (errno));
 		g_free (sig_fname);
 		return;
 	}
@@ -393,22 +433,23 @@ li_repository_sign (LiRepository *repo, const gchar *sigtext, GError **error)
 typedef struct {
 	LiRepository *repo;
 	GString *sigtext;
+	LiRepoIndexKinds ikind;
 
 	GError *error;
 } LiIndexSaveHelper;
 
 /**
- * li_repository_save_indices:
+ * li_repository_save_indices_helper:
  *
  * Helper function to save the package-index hashtable on disk
  */
 static void
-li_repository_save_indices (gchar *arch, LiPkgIndex *index, LiIndexSaveHelper *helper)
+li_repository_save_indices_helper (gchar *arch, LiPkgIndex *index, LiIndexSaveHelper *helper)
 {
-	_cleanup_free_ gchar *fname = NULL;
-	_cleanup_free_ gchar *dir = NULL;
-	_cleanup_free_ gchar *checksum = NULL;
-	_cleanup_free_ gchar *internal_name = NULL;
+	g_autofree gchar *fname = NULL;
+	g_autofree gchar *dir = NULL;
+	g_autofree gchar *checksum = NULL;
+	g_autofree gchar *internal_name = NULL;
 	LiRepositoryPrivate *priv = GET_PRIVATE (helper->repo);
 
 	/* don't continue if we already have an error */
@@ -418,7 +459,15 @@ li_repository_save_indices (gchar *arch, LiPkgIndex *index, LiIndexSaveHelper *h
 	dir = g_build_filename (priv->repo_path, "indices", arch, NULL);
 	g_mkdir_with_parents (dir, 0755);
 
-	fname = g_build_filename (dir, "Index.gz", NULL);
+	if (helper->ikind == LI_REPO_INDEX_KIND_COMMON)
+		fname = g_build_filename (dir, "Index.gz", NULL);
+	else if (helper->ikind == LI_REPO_INDEX_KIND_DEVEL)
+		fname = g_build_filename (dir, "Index-Devel.gz", NULL);
+	else if (helper->ikind == LI_REPO_INDEX_KIND_SOURCE)
+		fname = g_build_filename (dir, "Index-Sources.gz", NULL);
+	else
+		g_critical ("Handling unknown repository index type: %i", (gint) helper->ikind);
+
 	li_pkg_index_save_to_file (index, fname);
 
 	if (g_str_has_prefix (fname, priv->repo_path)) {
@@ -440,6 +489,39 @@ li_repository_save_indices (gchar *arch, LiPkgIndex *index, LiIndexSaveHelper *h
 	g_string_append_printf (helper->sigtext, "%s\t%s\n", checksum, internal_name);
 }
 
+/**
+ * li_repository_save_indices:
+ */
+static gboolean
+li_repository_save_indices (LiRepository *repo, GString *sigtext, LiRepoIndexKinds kind, GError **error)
+{
+	LiIndexSaveHelper helper;
+	GHashTable *index_tab;
+	LiRepositoryPrivate *priv = GET_PRIVATE (repo);
+
+	index_tab = g_hash_table_lookup (priv->indices,
+					li_repo_index_kind_to_string (kind));
+
+	/* do we have indices of this kind to save? */
+	if (index_tab == NULL)
+		return TRUE;
+
+	helper.repo = repo;
+	helper.sigtext = sigtext;
+	helper.error = NULL;
+	helper.ikind = kind;
+
+	g_hash_table_foreach (index_tab,
+				(GHFunc) li_repository_save_indices_helper,
+				&helper);
+	if (helper.error != NULL) {
+		g_propagate_error (error, helper.error);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 /**
  * li_repository_save_asmeta:
  *
@@ -448,10 +530,10 @@ li_repository_save_indices (gchar *arch, LiPkgIndex *index, LiIndexSaveHelper *h
 static void
 li_repository_save_asmeta (gchar *arch, AsMetadata *metad, LiIndexSaveHelper *helper)
 {
-	_cleanup_free_ gchar *fname = NULL;
-	_cleanup_free_ gchar *dir = NULL;
-	_cleanup_free_ gchar *checksum = NULL;
-	_cleanup_free_ gchar *internal_name = NULL;
+	g_autofree gchar *fname = NULL;
+	g_autofree gchar *dir = NULL;
+	g_autofree gchar *checksum = NULL;
+	g_autofree gchar *internal_name = NULL;
 	LiRepositoryPrivate *priv = GET_PRIVATE (helper->repo);
 
 	/* don't continue if we already have an error */
@@ -495,13 +577,10 @@ li_repository_save (LiRepository *repo, GError **error)
 {
 	gchar *dir;
 	GError *tmp_error = NULL;
+	g_autoptr(GString) sigtext = NULL;
 	LiIndexSaveHelper helper;
-	_cleanup_string_free_ GString *sigtext = NULL;
 	LiRepositoryPrivate *priv = GET_PRIVATE (repo);
 
-	/* prepare variable to store checksums */
-	sigtext = g_string_new ("");
-
 	/* ensure the basic directory structure is present */
 	dir = g_build_filename (priv->repo_path, "indices", NULL);
 	g_mkdir_with_parents (dir, 0755);
@@ -519,23 +598,30 @@ li_repository_save (LiRepository *repo, GError **error)
 	g_mkdir_with_parents (dir, 0755);
 	g_free (dir);
 
-	/* save indices */
-	helper.repo = repo;
-	helper.sigtext = sigtext;
-	helper.error = NULL;
+	/* prepare variable to store checksums */
+	sigtext = g_string_new ("");
 
-	g_hash_table_foreach (priv->indices,
-						(GHFunc) li_repository_save_indices,
-						&helper);
-	if (helper.error != NULL) {
-		g_propagate_error (error, helper.error);
+	/* save indices */
+	li_repository_save_indices (repo, sigtext, LI_REPO_INDEX_KIND_COMMON, &tmp_error);
+	if (tmp_error != NULL) {
+		g_propagate_error (error, tmp_error);
+		return FALSE;
+	}
+	li_repository_save_indices (repo, sigtext, LI_REPO_INDEX_KIND_DEVEL, &tmp_error);
+	if (tmp_error != NULL) {
+		g_propagate_error (error, tmp_error);
 		return FALSE;
 	}
 
 	/* save AppStream metadata */
+	helper.repo = repo;
+	helper.sigtext = sigtext;
+	helper.error = NULL;
+	helper.ikind = LI_REPO_INDEX_KIND_NONE;
+
 	g_hash_table_foreach (priv->asmeta,
-						(GHFunc) li_repository_save_asmeta,
-						&helper);
+				(GHFunc) li_repository_save_asmeta,
+				&helper);
 	if (helper.error != NULL) {
 		g_propagate_error (error, helper.error);
 		return FALSE;
@@ -558,7 +644,7 @@ gboolean
 li_repository_add_package (LiRepository *repo, const gchar *pkg_fname, GError **error)
 {
 	GError *tmp_error = NULL;
-	_cleanup_object_unref_ LiPackage *pkg = NULL;
+	g_autoptr(LiPackage) pkg = NULL;
 	LiPkgInfo *pki;
 	const gchar *pkgname;
 	const gchar *pkgversion;
@@ -566,12 +652,12 @@ li_repository_add_package (LiRepository *repo, const gchar *pkg_fname, GError **
 	gchar *tmp;
 	gunichar c;
 	guint i;
-	_cleanup_free_ gchar *dest_path = NULL;
-	_cleanup_free_ gchar *icon_dir = NULL;
-	_cleanup_free_ gchar *hash = NULL;
-	AsComponent *cpt;
+	g_autofree gchar *dest_path = NULL;
+	g_autofree gchar *icon_dir = NULL;
+	g_autofree gchar *hash = NULL;
 	LiPkgIndex *index;
 	AsMetadata *metad;
+	LiPackageKind kind;
 	LiRepositoryPrivate *priv = GET_PRIVATE (repo);
 
 	pkg = li_package_new ();
@@ -590,9 +676,9 @@ li_repository_add_package (LiRepository *repo, const gchar *pkg_fname, GError **
 		 * the packages signature and make it invalid.
 		 */
 		g_set_error (error,
-				LI_REPOSITORY_ERROR,
-				LI_REPOSITORY_ERROR_EMBEDDED_COPY,
-				_("The package contains embedded dependencies. Packages with that property are not allowed in repositories, please add dependencies separately."));
+			LI_REPOSITORY_ERROR,
+			LI_REPOSITORY_ERROR_EMBEDDED_COPY,
+			_("The package contains embedded dependencies. Packages with that property are not allowed in repositories, please add dependencies separately."));
 		return FALSE;
 	}
 
@@ -602,6 +688,7 @@ li_repository_add_package (LiRepository *repo, const gchar *pkg_fname, GError **
 	pkgname = li_pkg_info_get_name (pki);
 	pkgversion = li_pkg_info_get_version (pki);
 	pkgarch = li_pkg_info_get_architecture (pki);
+	kind = li_pkg_info_get_kind (pki);
 	tmp = g_str_to_ascii (pkgname, NULL);
 
 	for (i = 0; tmp[i] != '\0'; i++) {
@@ -613,10 +700,10 @@ li_repository_add_package (LiRepository *repo, const gchar *pkg_fname, GError **
 	c = g_unichar_tolower (c);
 
 	dest_path = g_strdup_printf ("pool/%c/%s-%s_%s.ipk",
-							c,
-							pkgname,
-							pkgversion,
-							pkgarch);
+					c,
+					pkgname,
+					pkgversion,
+					pkgarch);
 	tmp = g_strdup_printf ("%s/pool/%c/", priv->repo_path, c);
 	g_mkdir_with_parents (tmp, 0755);
 	g_free (tmp);
@@ -640,10 +727,10 @@ li_repository_add_package (LiRepository *repo, const gchar *pkg_fname, GError **
 
 	/* extract the icons */
 	icon_dir = g_build_filename (priv->repo_path,
-									"assets",
-									li_pkg_info_get_id (pki),
-									"icons",
-									NULL);
+					"assets",
+					li_pkg_info_get_name (pki),
+					"icons",
+					NULL);
 	li_package_extract_appstream_icons (pkg, icon_dir, &tmp_error);
 	if (tmp_error != NULL) {
 		g_free (tmp);
@@ -660,27 +747,37 @@ li_repository_add_package (LiRepository *repo, const gchar *pkg_fname, GError **
 	}
 	g_free (tmp);
 
-	cpt = li_package_get_appstream_cpt (pkg);
-	/* set an icon name */
-	if (g_file_test (icon_dir, G_FILE_TEST_EXISTS)) {
-		tmp = g_strdup_printf ("%s.png", li_pkg_info_get_id (pki));
-		/* TODO: Determine which sizes we exported, and set that information correctly */
-		as_component_add_icon (cpt, AS_ICON_KIND_CACHED, 0, 0, tmp);
-		g_free (tmp);
-	}
-
-	/* set a unique AppStream bundle name */
-	as_component_add_bundle_id (cpt, AS_BUNDLE_KIND_LIMBA,
-							li_pkg_info_get_id (pki));
-	/* remove all package names - just in case */
-	as_component_set_pkgnames (cpt, NULL);
-
 	/* add to indices */
-	index = li_repository_get_index (repo, pkgarch);
+	if (li_pkg_info_get_kind (pki) == LI_PACKAGE_KIND_DEVEL)
+		index = li_repository_get_index (repo, LI_REPO_INDEX_KIND_DEVEL, pkgarch);
+	else
+		index = li_repository_get_index (repo, LI_REPO_INDEX_KIND_COMMON, pkgarch);
 	li_pkg_index_add_package (index, pki);
 
-	metad = li_repository_get_asmeta (repo, pkgarch);
-	as_metadata_add_component (metad, cpt);
+	/* don't add to AppStream index, development packages don't belong there */
+	if (kind != LI_PACKAGE_KIND_DEVEL) {
+		AsComponent *cpt;
+
+		cpt = li_package_get_appstream_cpt (pkg);
+		/* set an icon name */
+		if (g_file_test (icon_dir, G_FILE_TEST_EXISTS)) {
+			tmp = g_strdup_printf ("%s-%s.png",
+						li_pkg_info_get_name (pki),
+						li_pkg_info_get_version (pki));
+			/* TODO: Determine which sizes we exported, and set that information correctly */
+			as_component_add_icon (cpt, AS_ICON_KIND_CACHED, 0, 0, tmp);
+			g_free (tmp);
+		}
+
+		/* set a unique AppStream bundle name */
+		as_component_add_bundle_id (cpt, AS_BUNDLE_KIND_LIMBA,
+						li_pkg_info_get_id (pki));
+		/* remove all package names - just in case */
+		as_component_set_pkgnames (cpt, NULL);
+
+		metad = li_repository_get_asmeta (repo, pkgarch);
+		as_metadata_add_component (metad, cpt);
+	}
 
 	return TRUE;
 }
@@ -696,7 +793,7 @@ li_repository_find_icons (const gchar* repo_dir, const gchar *icon_size, GError
 	GFileEnumerator *enumerator = NULL;
 	GFile *fdir;
 	GPtrArray *icon_paths = NULL;
-	_cleanup_free_ gchar *asset_dir = NULL;
+	g_autofree gchar *asset_dir = NULL;
 
 	if (!g_file_test (repo_dir, G_FILE_TEST_EXISTS))
 		return NULL;
@@ -715,7 +812,7 @@ li_repository_find_icons (const gchar* repo_dir, const gchar *icon_size, GError
 	icon_paths = g_ptr_array_new_with_free_func (g_free);
 
 	while ((file_info = g_file_enumerator_next_file (enumerator, NULL, &tmp_error)) != NULL) {
-		_cleanup_free_ gchar *path = NULL;
+		g_autofree gchar *path = NULL;
 
 		if (tmp_error != NULL) {
 			g_propagate_error (error, tmp_error);
@@ -725,11 +822,11 @@ li_repository_find_icons (const gchar* repo_dir, const gchar *icon_size, GError
 		if (g_file_info_get_is_hidden (file_info))
 			continue;
 		path = g_build_filename (asset_dir,
-								g_file_info_get_name (file_info),
-								NULL);
+					 g_file_info_get_name (file_info),
+					 NULL);
 
 		if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
-			_cleanup_free_ gchar *icon_dir = NULL;
+			g_autofree gchar *icon_dir = NULL;
 			GPtrArray *png_files;
 			guint i;
 
@@ -771,8 +868,8 @@ li_repository_create_icon_tarball (LiRepository *repo, const gchar *icon_size, G
 	int len;
 	int fd;
 	guint i;
-	_cleanup_ptrarray_unref_ GPtrArray *files = NULL;
-	_cleanup_free_ gchar *tarball_fname = NULL;
+	g_autoptr(GPtrArray) files = NULL;
+	g_autofree gchar *tarball_fname = NULL;
 	gchar *tmp;
 	GError *tmp_error = NULL;
 	LiRepositoryPrivate *priv = GET_PRIVATE (repo);
