@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2009-2014 Matthias Klumpp <matthias@tenstral.net>
  *
@@ -33,6 +33,7 @@
 #include <sys/types.h>
 #include <sys/utsname.h>
 #include <errno.h>
+#include "li-systemd-dbus.h"
 
 /**
  * SECTION:li-utils
@@ -278,29 +279,29 @@ li_str_replace (const gchar *str, const gchar *old, const gchar *new)
 {
 	gchar *ret, *r;
 	const gchar *p, *q;
-	size_t oldlen = strlen(old);
-	size_t count, retlen, newlen = strlen(new);
+	size_t oldlen = strlen (old);
+	size_t count, retlen, newlen = strlen (new);
 
 	if (oldlen != newlen) {
-		for (count = 0, p = str; (q = strstr(p, old)) != NULL; p = q + oldlen)
+		for (count = 0, p = str; (q = strstr (p, old)) != NULL; p = q + oldlen)
 			count++;
 		/* this is undefined if p - str > PTRDIFF_MAX */
-		retlen = p - str + strlen(p) + count * (newlen - oldlen);
+		retlen = p - str + strlen (p) + count * (newlen - oldlen);
 	} else
-		retlen = strlen(str);
+		retlen = strlen (str);
 
-	if ((ret = malloc(retlen + 1)) == NULL)
+	if ((ret = malloc (retlen + 1)) == NULL)
 		return NULL;
 
 	for (r = ret, p = str; (q = strstr(p, old)) != NULL; p = q + oldlen) {
 		/* this is undefined if q - p > PTRDIFF_MAX */
 		ptrdiff_t l = q - p;
-		memcpy(r, p, l);
+		memcpy (r, p, l);
 		r += l;
-		memcpy(r, new, newlen);
+		memcpy (r, new, newlen);
 		r += newlen;
 	}
-	strcpy(r, p);
+	strcpy (r, p);
 
 	return ret;
 }
@@ -314,7 +315,7 @@ gchar*
 li_compute_checksum_for_file (const gchar *fname)
 {
 	const GChecksumType cstype = G_CHECKSUM_SHA256;
-	_cleanup_checksum_free_ GChecksum *cs;
+	g_autoptr(GChecksum) cs;
 	guchar data[4096] = {0};
 	size_t size = 0;
 	FILE *input;
@@ -377,6 +378,10 @@ li_utils_get_tmp_dir (const gchar *prefix)
 	gchar *tmp_dir = NULL;
 	const gchar *tmp_root_path = "/var/tmp/limba";
 
+	/* ensure nobody symlinked this directory to a different location */
+	if (g_file_test (tmp_root_path, G_FILE_TEST_IS_SYMLINK))
+		g_assert (g_unlink (tmp_root_path) == 0);
+
 	g_mkdir_with_parents (tmp_root_path, 0777);
 
 	template = g_strdup_printf ("%s-XXXXXX", prefix);
@@ -392,6 +397,9 @@ li_utils_get_tmp_dir (const gchar *prefix)
 	tmp_dir = g_strdup (tmp_dir);
 	g_free (path);
 
+	/* ensure that the permissions on the root path haven't changed */
+	g_chmod (tmp_root_path, 0777);
+
 	return tmp_dir;
 }
 
@@ -618,7 +626,7 @@ li_compare_versions (const gchar* a, const gchar *b)
 LiPkgInfo*
 li_parse_dependency_string (const gchar *depstr)
 {
-	_cleanup_free_ gchar *dep_raw = NULL;
+	g_autofree gchar *dep_raw = NULL;
 	LiPkgInfo *pki;
 
 	dep_raw = g_strdup (depstr);
@@ -704,3 +712,204 @@ li_parse_dependencies_string (const gchar *depstr)
 
 	return array;
 }
+
+/**
+ * li_env_get_user_fullname:
+ *
+ * Get the user's full name from the environment.
+ *
+ * Returns: User's full name, or %NULL on error. Free with g_free()
+ */
+gchar*
+li_env_get_user_fullname (void)
+{
+	const gchar *var;
+
+	var = g_getenv ("LIMBA_FULLNAME");
+	if (var == NULL)
+		var = g_getenv ("DEBFULLNAME");
+	if (var == NULL)
+		return NULL;
+	else
+		return g_strdup (var);
+}
+
+/**
+ * li_env_get_user_email:
+ *
+ * Get the user's email address from the environment.
+ *
+ * Returns: Email address, or %NULL on error. Free with g_free()
+ */
+gchar*
+li_env_get_user_email (void)
+{
+	const gchar *var;
+
+	var = g_getenv ("LIMBA_EMAIL");
+	if (var == NULL)
+		var = g_getenv ("DEBEMAIL");
+	if (var == NULL)
+		return NULL;
+	else
+		return g_strdup (var);
+}
+
+/**
+ * li_env_get_target_repo:
+ *
+ * Get the target repository name.
+ *
+ * Returns: Target repository name, or %NULL on error. Free with g_free()
+ */
+gchar*
+li_env_get_target_repo (void)
+{
+	const gchar *var;
+	var = g_getenv ("LIMBA_TARGET_REPO");
+	if (var == NULL)
+		return NULL;
+	else
+		return g_strdup (var);
+}
+
+/**
+ * li_env_set_user_details:
+ *
+ * Set user details used when building a Limba package
+ */
+void
+li_env_set_user_details (const gchar *user_name, const gchar *user_email, const gchar *target_repo)
+{
+	if (user_name != NULL)
+		g_setenv ("LIMBA_FULLNAME", user_name, TRUE);
+	if (user_email != NULL)
+		g_setenv ("LIMBA_EMAIL", user_email, TRUE);
+	if (target_repo != NULL)
+		g_setenv ("LIMBA_TARGET_REPO", target_repo, TRUE);
+}
+
+/**
+ * SdJobData:
+ *
+ * Helper data for systemd job callback
+ */
+struct SdJobData {
+	gchar *job;
+	GMainLoop *main_loop;
+};
+
+/**
+ * li_sd_job_removed_cb:
+ *
+ * Helper callback
+ */
+static void
+li_sd_job_removed_cb (LiSdManager *sdmgr, guint32 id, gchar *job, gchar *unit, gchar *result, struct SdJobData *data)
+{
+	if (strcmp (job, data->job) == 0)
+		g_main_loop_quit (data->main_loop);
+}
+
+/**
+ * li_add_to_new_scope:
+ *
+ * Add the current process to a new scope (cgroup).
+ */
+void
+li_add_to_new_scope (const gchar *domain, const gchar *idname, GError **error)
+{
+	GDBusConnection *conn = NULL;
+	LiSdManager *sdmgr = NULL;
+	GMainLoop *main_loop = NULL;
+	g_autofree gchar *sd_path = NULL;
+	g_autofree gchar *sd_address = NULL;
+	g_autofree gchar *sd_job = NULL;
+	g_autofree gchar *cgname = NULL;
+	GVariantBuilder builder;
+	GVariant *properties = NULL;
+	GVariant *aux = NULL;
+	guint32 pid;
+	GError *tmp_error = NULL;
+	struct SdJobData data;
+
+	if (domain == NULL)
+		domain = "limba";
+
+	/* check if systemd is running */
+	if (li_utils_is_root ()) {
+		sd_path = g_strdup ("/run/systemd/private");
+	} else {
+		sd_path = g_strdup_printf ("/run/user/%d/systemd/private", getuid ());
+	}
+	if (!g_file_test (sd_path, G_FILE_TEST_EXISTS))
+		goto out;
+
+	pid = getpid ();
+
+	main_loop = g_main_loop_new (NULL, FALSE);
+
+	sd_address = g_strconcat ("unix:path=", sd_path, NULL);
+	conn = g_dbus_connection_new_for_address_sync (sd_address,
+						G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT,
+						NULL,
+						NULL, &tmp_error);
+	if (tmp_error != NULL) {
+		g_propagate_prefixed_error (error, tmp_error,
+					"Unable to connect to systemd.");
+		goto out;
+	}
+
+	sdmgr = li_sd_manager_proxy_new_sync (conn,
+					G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
+					NULL,
+					"/org/freedesktop/systemd1",
+					NULL,
+					&tmp_error);
+	if (tmp_error != NULL) {
+		g_propagate_prefixed_error (error, tmp_error,
+					"Unable to create systemd manager proxy.");
+		goto out;
+	}
+
+	cgname = g_strdup_printf ("%s-%s-%d.scope", domain, idname, pid);
+
+	g_variant_builder_init (&builder, G_VARIANT_TYPE ("a(sv)"));
+	g_variant_builder_add (&builder, "(sv)",
+				"PIDs",
+				g_variant_new_fixed_array (G_VARIANT_TYPE ("u"),
+							&pid, 1, sizeof (guint32)));
+	properties = g_variant_builder_end (&builder);
+
+	aux = g_variant_new_array (G_VARIANT_TYPE ("(sa(sv))"), NULL, 0);
+	li_sd_manager_call_start_transient_unit_sync (sdmgr,
+						cgname, /* scope name */
+						"fail", /* mode */
+						properties,
+						aux, /* unused, empty array */
+						&sd_job,
+						NULL,
+						&tmp_error);
+	if (tmp_error != NULL) {
+		g_propagate_prefixed_error (error, tmp_error,
+					"Unable to create systemd manager proxy.");
+		goto out;
+	}
+
+	data.job = sd_job;
+	data.main_loop = main_loop;
+	g_signal_connect (sdmgr,
+			"job-removed",
+			G_CALLBACK (li_sd_job_removed_cb), &data);
+
+	/* wait for the job to complete */
+	g_main_loop_run (main_loop);
+
+out:
+	if (main_loop)
+		g_main_loop_unref (main_loop);
+	if (sdmgr)
+		g_object_unref (sdmgr);
+	if (conn)
+		g_object_unref (conn);
+}
